package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.Account
import com.merchantsofrome.server.model.object_parameters.LoginData
import com.merchantsofrome.server.model.Quest
import com.merchantsofrome.server.repository.*
import com.merchantsofrome.server.service.LoginService
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api")
class AccountController(private val accountRepository: AccountRepository, private val questRepository: QuestRepository,
                        private val itemAccountRepository: ItemAccountRepository, private val itemRepository: ItemRepository,
                        private val processAccountRepository: ProcessAccountRepository, private val processRepository: ProcessRepository,
                        private val plantAccountRepository: PlantAccountRepository, private val fortuneRepository: FortuneRepository) {

    fun MutableMap<String, Int>.addUserData(account: Account, data: Int){
        if(data<0){
            return
        }
        val name = account.alias ?: account.username
        this[name] = data
    }
    //TODO add validation and response
    @PostMapping("/createNewAccount")
    fun createNewAccount(@Valid @RequestBody loginData: LoginData): ResponseEntity<Boolean> {
        if (accountRepository.findById(loginData.username).isPresent) {
            return ResponseEntity(false, HttpStatus.CONFLICT)
        }
        //Account details; Username, pw and salt
        val salt = LoginService.generateSalt()
        val password = LoginService.getHashedPassword(loginData.password, salt)
        val newAccount = Account(username = loginData.username, password = password, salt = salt)
        accountRepository.save(newAccount)
        questRepository.save(Quest(account = newAccount, questId = 1, checkpointReached = 0))

        return ResponseEntity(true, HttpStatus.OK)
    }

    @GetMapping("/accounts")
    fun getUserById(@RequestHeader("Username") username: String): ResponseEntity<Account> {
        return accountRepository.findById(username).map { user ->
            ResponseEntity.ok(user)
        }.orElse(ResponseEntity.notFound().build())
    }

    @GetMapping("/accounts/rules")
    fun rulesAccepted(@RequestHeader("Username") username: String): ResponseEntity<Any>{
        val account = accountRepository.getByUsername(username)!!
        return ResponseEntity(object{val value: Boolean = account.rulesAccepted}, HttpStatus.OK)
    }

    @PostMapping("/accounts/rules")
    fun acceptRules(@RequestHeader("Username") username: String): Any {
        val account = accountRepository.getByUsername(username)!!
        accountRepository.save(account.copy(rulesAccepted = true))
        return ResponseEntity<Any>(HttpStatus.OK)
    }

    @PutMapping("/accounts/changepassword")
    fun changePassword(@RequestHeader("Username") username: String,
                       @Valid @RequestBody info: Array<String>): Any {
        val oldPassword: String = info[0]
        val account: Account = accountRepository.findById(username).get()
        val oldSalt: String = account.salt
        val oldHashedPassword: String = LoginService.getHashedPassword(password = oldPassword, salt = oldSalt)
        if (account.password == oldHashedPassword) {
            val newPassword: String = info[1]
            val salt: String = LoginService.generateSalt()
            val password: String = LoginService.getHashedPassword(password = newPassword, salt = salt)
            val newAccount: Account = account.copy(password = password, salt = salt)
            accountRepository.save(newAccount)
            return ResponseEntity<Any>(HttpStatus.OK)
        }
        return ResponseEntity<Any>(HttpStatus.CONFLICT)
    }

    @GetMapping("/accounts/scoreboard/coins")
    fun getScoreboardBasedOnGold(@RequestHeader("Username") username: String): ResponseEntity<MutableMap<String, Int>> {
        val score: MutableMap<String, Int> = HashMap()
        accountRepository.findAll().forEach {
            if (questRepository.getQuestByAccount_UsernameAndQuestId(it.username, 2)?.finished == true || it.username == username) {
                score.addUserData(it,it.coinsOwned)
            }

        }
        return ResponseEntity(score, HttpStatus.OK)
    }

    @GetMapping("/accounts/scoreboard/wheatHarvested")
    fun getScoreboardBasedOnWheatHarvested(@RequestHeader("Username") username: String): ResponseEntity<MutableMap<String, Int>> {
        val score: MutableMap<String, Int> = HashMap()
        accountRepository.findAll().forEach {
            if (questRepository.getQuestByAccount_UsernameAndQuestId(it.username, 2)?.finished == true || it.username == username) {
                score.addUserData(it,plantAccountRepository.getPlantAccountByAccount_UsernameAndPlantName(it.username, "wheat")!!.totalAmountHarvested)
            }

        }
        return ResponseEntity(score, HttpStatus.OK)
    }

    @GetMapping("/accounts/scoreboard/fortune")
    fun getScoreboardBasedOnFortune(@RequestHeader("Username") username: String): ResponseEntity<MutableMap<String, Int>> {
        val score: MutableMap<String, Int> = HashMap()
        accountRepository.findAll().forEach{
            if (questRepository.getQuestByAccount_UsernameAndQuestId(it.username, 2)?.finished == true || it.username == username) {
                score.addUserData(it, fortuneRepository.getByUsername(it.username)?.fortune ?: -1)
            }
        }
        return ResponseEntity(score, HttpStatus.OK)
    }



}