package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.*
import com.merchantsofrome.server.repository.*
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import javax.annotation.PostConstruct

@RestController
@RequestMapping("/api")
class DBController(private val itemRepository: ItemRepository, private val plantRepository: PlantRepository,
                   private val processRepository: ProcessRepository, private val oreRepository: OreRepository,
                   private val processItemRepository: ProcessItemRepository, private val versionRepository: VersionRepository,
                   private val treeRepository: TreeRepository, private val questRepository: QuestRepository,
                   private val accountRepository: AccountRepository, private val treePatchRepository: TreePatchRepository,
                   private val feedbackQuestionRepository: FeedbackQuestionRepository) {

    //Todo: Fix static setting of Id's so that this method can be updated without compromising the data and add @PostConstruct
    @PostMapping("/dbcontroller")
    fun addToDatabase(): ResponseEntity<Any> {
        addItems()
        addPlants()
        addOres()
        addTrees()
        addProcesses()
        addFeedbackQuestions()
        addNewQuests(maxQuest = 3)
        validateLastTreePatchUnlocked()
        return ResponseEntity("Database updated", HttpStatus.OK)
    }

    private fun addFeedbackQuestions(){
        var index = 0
        // After quest 1
        addFeedbackQuestion(index++, 0, "Thank you for playing Merchants of Rome~As you " +
                "might know this game was made as part of a bachelor's thesis and as such, we would appreciate" +
                " if you could help us out by giving us some feedback~As a thank you for your cooperation, you will be" +
                " awarded 1 coin once you have answered all questions","info")
        addFeedbackQuestion(index++, 0, "Some specific questions first","info")
        addFeedbackQuestion(index++, 0,"How did you do in the beginning of the game?~" +
                "Did you feel that it was hard to get started?~Was there anything that you struggled with?", "text")
        addFeedbackQuestion(index++, 0, "Did you have any trouble selling your bread to the villagers?", "text")
        addFeedbackQuestion(index++, 0, "Some general questions", "info")
        addFeedbackQuestion(index++, 0, "Is there anything in particular you like about the game?", "text")
        addFeedbackQuestion(index++, 0, "Is there anything in particular you dislike about the game?", "text")
        addFeedbackQuestion(index++, 0, "If you could make some changes and/or additions to the game, what would you do?", "text")
        addFeedbackQuestion(index++, 0, "Thanks for your help :)~1 coin has been added to your account","end")

        // After quest 3
        addFeedbackQuestion(index++, 1, "Thank you for playing Merchants of Rome~As you " +
                "might know this game was made as part of a bachelor's thesis and as such, we would appreciate" +
                " if you could help us out by giving us some feedback~As a thank you for your cooperation, you will be" +
                " awarded 100 coins once you have answered all questions","info")
        addFeedbackQuestion(index++, 1, "What do you think of the game so far?~Does the pacing feel ok?","text")
        addFeedbackQuestion(index++, 1, "Is there anything in particular that you like about the game?","text")
        addFeedbackQuestion(index++, 1, "Is there anything in particular that you dislike about the game?","text")
        addFeedbackQuestion(index++, 1, "If you could make some changes and/or additions to the game, what would you do?","text")
        addFeedbackQuestion(index++, 1, "Do you feel that the different parts of the game are well balanced?","text")
        addFeedbackQuestion(index, 1, "Thanks for your help :)~100 coins has been added to your account","end")
    }

    private fun addFeedbackQuestion(id: Int, questionSetId: Int, questionText: String, questionType: String){
        val oldFQ = feedbackQuestionRepository.findById(id)
        if (oldFQ.isPresent) {
            feedbackQuestionRepository.delete(oldFQ.get())
        }
        val fq = FeedbackQuestion(id = id, questionSetId = questionSetId, questionText = questionText, questionType = questionType)
        feedbackQuestionRepository.save(fq)
    }

    private fun validateLastTreePatchUnlocked(){
        val users = accountRepository.findAll()
        users.forEach {
            val treeAccounts = treePatchRepository.getTreePatchesByAccount_Username(it.username);
            if(treeAccounts.isEmpty()){
                accountRepository.save(it.copy(indexOfLastTreePatchUnlocked = -1))
            }
        }
    }

    private fun addNewQuests(maxQuest: Int) {
        val users = accountRepository.findAll()
        users.forEach {
            for (i in 1..(maxQuest-1)) {
                val prevQuest = questRepository.getQuestByAccount_UsernameAndQuestId(it.username, i)
                val nextQuest = questRepository.getQuestByAccount_UsernameAndQuestId(it.username, i + 1)
                if (prevQuest == null) {
                    break
                }
                if (prevQuest.finished && nextQuest == null) {
                    questRepository.save(Quest(questId = i+1, account = it, checkpointReached = 0, finished = false))
                }
            }
        }
    }

    private fun addItems() {
        // Neste id: 35
        addItem(itemId = 0, itemCategory = Category.crops, itemName = "wheat", itemPrice = -1)
        addItem(itemId = 1, itemCategory = Category.crops, itemName = "barley", itemPrice = -1)
        addItem(itemId = 2, itemCategory = Category.crops, itemName = "grapes", itemPrice = -1)
        addItem(itemId = 3, itemCategory = Category.crops, itemName = "olives", itemPrice = -1)
        addItem(itemId = 4, itemCategory = Category.crops, itemName = "cucumber", itemPrice = 5) // 16.67 gold per hour
        addItem(itemId = 5, itemCategory = Category.crops, itemName = "lettuce", itemPrice = 2) // 14 gold per hour
        addItem(itemId = 29, itemCategory = Category.crops, itemName = "carrot", itemPrice = 7) // 18.67 gold per hour

        addItem(itemId = 6, itemCategory = Category.ores, itemName = "stone", itemPrice = -1)
        addItem(itemId = 7, itemCategory = Category.ores, itemName = "clay", itemPrice = -1)
        addItem(itemId = 8, itemCategory = Category.ores, itemName = "ironOre", itemPrice = -1)
        addItem(itemId = 9, itemCategory = Category.ores, itemName = "copperOre", itemPrice = -1)
        addItem(itemId = 10, itemCategory = Category.ores, itemName = "silverOre", itemPrice = -1)
        addItem(itemId = 11, itemCategory = Category.ores, itemName = "goldOre", itemPrice = -1)

        addItem(itemId = 12, itemCategory = Category.bars, itemName = "iron", itemPrice = 10)
        addItem(itemId = 13, itemCategory = Category.bars, itemName = "copper", itemPrice = 4)
        addItem(itemId = 14, itemCategory = Category.bars, itemName = "silver", itemPrice = 7)
        addItem(itemId = 15, itemCategory = Category.bars, itemName = "gold", itemPrice = 10)

        addItem(itemId = 16, itemCategory = Category.items, itemName = "sword", itemPrice = 50)
        addItem(itemId = 17, itemCategory = Category.items, itemName = "fishHooks", itemPrice = 1)
        addItem(itemId = 18, itemCategory = Category.items, itemName = "boat", itemPrice = 1000)
        addItem(itemId = 19, itemCategory = Category.items, itemName = "logs", itemPrice = 10)
        addItem(itemId = 20, itemCategory = Category.items, itemName = "planks", itemPrice = 5)
        addItem(itemId = 21, itemCategory = Category.items, itemName = "nails", itemPrice = 1)

        addItem(itemId = 22, itemCategory = Category.food, itemName = "bread", itemPrice = 2) // 8 gold per hour
        addItem(itemId = 23, itemCategory = Category.food, itemName = "flour", itemPrice = -1)
        addItem(itemId = 24, itemCategory = Category.food, itemName = "wine", itemPrice = 3)
        addItem(itemId = 25, itemCategory = Category.food, itemName = "oliveOil", itemPrice = 3)
        addItem(itemId = 26, itemCategory = Category.food, itemName = "fish", itemPrice = 4)
        addItem(itemId = 30, itemCategory = Category.food, itemName = "barleyFlour", itemPrice = -1)
        addItem(itemId = 31, itemCategory = Category.food, itemName = "wholeGrainBread", itemPrice = 7) //18.9 gold per hour
        addItem(itemId = 32, itemCategory = Category.food, itemName = "multiGrainBread", itemPrice = 6) //22.32 gold per hour

        addItem(itemId = 27, itemCategory = Category.fruits, itemName = "apple", itemPrice = 3) // 15 gold per hour
        addItem(itemId = 28, itemCategory = Category.fruits, itemName = "peach", itemPrice = 5) // 18.75 gold per hour
        addItem(itemId = 33, itemCategory = Category.fruits, itemName = "orange", itemPrice = 2) // 22 gold per hour
        addItem(itemId = 34, itemCategory = Category.fruits, itemName = "chestnut", itemPrice = 8) // 20 gold per hour
    }

    private fun addItem(itemId: Int, itemCategory: String, itemName: String, itemPrice: Int) {
        val oldItem = itemRepository.findById(itemId)
        if (oldItem.isPresent) {
            itemRepository.delete(oldItem.get())
        }
        val item = Item(id = itemId, category = itemCategory, name = itemName, price = itemPrice)
        itemRepository.save(item)
    }

    private fun addPlants() {
        addPlant(plantName = "wheat", growthTimeInMillies = minutes(1), amountWhenFullyGrownMax = 60, amountWhenFullyGrownMin = 30)
        addPlant(plantName = "lettuce", growthTimeInMillies = hours(1), amountWhenFullyGrownMax = 10, amountWhenFullyGrownMin = 4)
        addPlant(plantName = "cucumber", growthTimeInMillies = hours(6), amountWhenFullyGrownMax = 26, amountWhenFullyGrownMin = 14)
        addPlant(plantName = "barley", growthTimeInMillies = minutes(5), amountWhenFullyGrownMax = 300, amountWhenFullyGrownMin = 150)
        addPlant(plantName = "carrot", growthTimeInMillies = hours(3), amountWhenFullyGrownMax = 11, amountWhenFullyGrownMin = 5)
        addPlant(plantName = "grapes", growthTimeInMillies = hours(1000), amountWhenFullyGrownMax = 25, amountWhenFullyGrownMin = 5)
        addPlant(plantName = "olives", growthTimeInMillies = hours(1000), amountWhenFullyGrownMax = 30, amountWhenFullyGrownMin = 20)

    }

    private fun addPlant(plantName: String, growthTimeInMillies: Int, amountWhenFullyGrownMax: Int, amountWhenFullyGrownMin: Int) {
        val oldPlant = plantRepository.findById(plantName)
        if (oldPlant.isPresent) {
            plantRepository.delete(oldPlant.get())
        }
        val plant = Plant(name = plantName, growthTimeInMillies = growthTimeInMillies, amountWhenFullyGrownMax = amountWhenFullyGrownMax, amountWhenFullyGrownMin = amountWhenFullyGrownMin)
        plantRepository.save(plant)
    }

    private fun addTrees() {
        addTree(name = "appleTree", growthTimeInMillis = hours(12), growthTimeFruitInMillis = hours(1), amountLogsWhenFullyGrownMax = 20, amountLogsWhenFullyGrownMin = 10, amountFruitWhenFullyGrownMax = 7, amountFruitWhenFullyGrownMin = 3, fruitId = itemRepository.getItemByName("apple")!!.id) // 12.5 gold per hour
        addTree(name = "peachTree", growthTimeInMillis = hours(16), growthTimeFruitInMillis = hours(2), amountLogsWhenFullyGrownMax = 30, amountLogsWhenFullyGrownMin = 20, amountFruitWhenFullyGrownMax = 9, amountFruitWhenFullyGrownMin = 6, fruitId = itemRepository.getItemByName("peach")!!.id) // 15.625 gold per hour
        addTree(name = "orangeTree", growthTimeInMillis = hours(24), growthTimeFruitInMillis = minutes(30), amountLogsWhenFullyGrownMax = 50, amountLogsWhenFullyGrownMin = 36, amountFruitWhenFullyGrownMax = 7, amountFruitWhenFullyGrownMin = 4, fruitId = itemRepository.getItemByName("orange")!!.id) // 17.9 gold per hour
        addTree(name = "chestnutTree", growthTimeInMillis = hours(8), growthTimeFruitInMillis = hours(4), amountLogsWhenFullyGrownMax = 21, amountLogsWhenFullyGrownMin = 14, amountFruitWhenFullyGrownMax = 13, amountFruitWhenFullyGrownMin = 7, fruitId = itemRepository.getItemByName("chestnut")!!.id) // 21.875 gold per hour
    }

    private fun addTree(name: String, growthTimeInMillis: Int, growthTimeFruitInMillis: Int, amountLogsWhenFullyGrownMax: Int, amountLogsWhenFullyGrownMin: Int, amountFruitWhenFullyGrownMax: Int, amountFruitWhenFullyGrownMin: Int, fruitId: Int) {
        val oldTree = treeRepository.findById(name)
        if (oldTree.isPresent) {
            treeRepository.delete(oldTree.get())
        }
        val tree = Tree(name = name, growthTimeInMillis = growthTimeInMillis, growthTimeFruitInMillis = growthTimeFruitInMillis, amountLogsWhenFullyGrownMax = amountLogsWhenFullyGrownMax, amountLogsWhenFullyGrownMin = amountLogsWhenFullyGrownMin, amountFruitWhenFullyGrownMax = amountFruitWhenFullyGrownMax, amountFruitWhenFullyGrownMin = amountFruitWhenFullyGrownMin, fruitId = fruitId)
        treeRepository.save(tree)
    }

    private fun addOres() {
        addOre(oreId = 0, oreName = "stone", minAmountPerSec = 2, maxAmountPerSec = 4)
        addOre(oreId = 1, oreName = "clay", minAmountPerSec = 3, maxAmountPerSec = 5)
        addOre(oreId = 2, oreName = "iron", minAmountPerSec = 2, maxAmountPerSec = 3)
        addOre(oreId = 3, oreName = "copper", minAmountPerSec = 2, maxAmountPerSec = 3)
        addOre(oreId = 4, oreName = "silver", minAmountPerSec = 1, maxAmountPerSec = 2)
        addOre(oreId = 5, oreName = "gold", minAmountPerSec = 0, maxAmountPerSec = 1)
    }

    private fun addOre(oreId: Int, oreName: String, minAmountPerSec: Int, maxAmountPerSec: Int) {
        val oldOre = oreRepository.findById(oreId)
        if (oldOre.isPresent) {
            oreRepository.delete(oldOre.get())
        }
        val ore = Ore(oreId = oreId, oreName = oreName, minAmountPerSec = minAmountPerSec, maxAmountPerSec = maxAmountPerSec)
        oreRepository.save(ore)
    }

    private fun addProcesses() {
        addProcess(processId = 0, nameOfProcess = "wheatFlour", durationForProcess = seconds(30), categoryForProcess = "mill", nameItemsIn = listOf(Pair("wheat", 20)), nameItemsOut = listOf(Pair("flour", 1)))  // wheat -> flour
        addProcess(processId = 1, nameOfProcess = "flourBread", durationForProcess = minutes(3), categoryForProcess = "baker", nameItemsIn = listOf(Pair("flour", 30)), nameItemsOut = listOf(Pair("bread", 1)))  // flour -> bread
        addProcess(processId = 2, nameOfProcess = "smeltIronOre", durationForProcess = minutes(10), categoryForProcess = "foundry", nameItemsIn = listOf(Pair("ironOre", 100)), nameItemsOut = listOf(Pair("iron", 1)))  // iron ore -> iron
        addProcess(processId = 3, nameOfProcess = "ironSword", durationForProcess = hours(1), categoryForProcess = "forge", nameItemsIn = listOf(Pair("iron", 4)), nameItemsOut = listOf(Pair("sword", 1)))  // iron -> sword
        addProcess(processId = 4, nameOfProcess = "ironFishHooks", durationForProcess = minutes(5), categoryForProcess = "forge", nameItemsIn = listOf(Pair("iron", 1)), nameItemsOut = listOf(Pair("fishHooks", 5)))  // iron -> fish hooks
        addProcess(processId = 5, nameOfProcess = "logsPlanks", durationForProcess = minutes(15), categoryForProcess = "woodWorker", nameItemsIn = listOf(Pair("logs", 1)), nameItemsOut = listOf(Pair("planks", 2)))  // timber -> wood planks
        addProcess(processId = 6, nameOfProcess = "planksBoat", durationForProcess = hours(2), categoryForProcess = "boatbuilder", nameItemsIn = listOf(Pair("planks", 150), Pair("nails", 300)), nameItemsOut = listOf(Pair("boat", 1)))  // wood planks && nails -> boat
        addProcess(processId = 7, nameOfProcess = "ironNails", durationForProcess = minutes(15), categoryForProcess = "forge", nameItemsIn = listOf(Pair("iron", 1)), nameItemsOut = listOf(Pair("nails", 5)))  // iron -> nails
        addProcess(processId = 8, nameOfProcess = "barleyBarleyFlour", durationForProcess = minutes(1), categoryForProcess = "mill", nameItemsIn = listOf(Pair("barley", 50)), nameItemsOut = listOf(Pair("barleyFlour", 1)))  // barley -> barleyFlour
        addProcess(processId = 9, nameOfProcess = "barleyFlourWholeGrainBread", durationForProcess = minutes(5), categoryForProcess = "baker", nameItemsIn = listOf(Pair("barleyFlour", 20)), nameItemsOut = listOf(Pair("wholeGrainBread", 1)))  // barleyFlour -> wholeGrainBread
        addProcess(processId = 10, nameOfProcess = "floursMultiGrainBread", durationForProcess = minutes(6), categoryForProcess = "baker", nameItemsIn = listOf(Pair("barleyFlour", 10), Pair("flour", 10)), nameItemsOut = listOf(Pair("multiGrainBread", 1)))  // barleyFlour -> multiGrainBread
    }

    private fun addProcess(processId: Int, nameOfProcess: String, durationForProcess: Int, categoryForProcess: String, nameItemsIn: List<Pair<String, Int>>, nameItemsOut: List<Pair<String, Int>>) {
        val oldProcess = processRepository.findById(processId)
        if (oldProcess.isPresent) {
            processRepository.delete(oldProcess.get())
        }
        val process = Process(id = processId, name = nameOfProcess, duration = durationForProcess.toLong(), category = categoryForProcess)
        for (pair: Pair<String, Int> in nameItemsIn) {
            val itemIn: Item = itemRepository.getItemByName(pair.first)!!
            val input = ProcessItem(processId = process.id, itemId = itemIn.id, amount = pair.second)
            processItemRepository.save(input)
            process.input.add(element = input)
        }
        for (pair: Pair<String, Int> in nameItemsOut) {
            val itemOut: Item = itemRepository.getItemByName(pair.first)!!
            val output = ProcessItem(processId = process.id, itemId = itemOut.id, amount = pair.second)
            processItemRepository.save(output)
            process.output.add(element = output)
        }
        processRepository.save(process)
    }

    @PostConstruct
    fun setVersion() {
        val version = versionRepository.getById(0)
        if (version != null) {
            versionRepository.updateVersion(version.version + 1)
        } else versionRepository.save(Version(0, 0))
    }

    private fun seconds(seconds: Int): Int = seconds * 1000
    private fun minutes(minutes: Int): Int = minutes * 60 * seconds(1)
    private fun hours(hours: Int): Int = hours * 60 * minutes(1)
    private fun time(hours: Int = 0, minutes: Int = 0, seconds: Int = 1): Int = hours(hours) + minutes(minutes) + seconds(seconds)

}