package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.*
import com.merchantsofrome.server.model.object_parameters.FarmPlotPlanting
import com.merchantsofrome.server.repository.*
import org.joda.time.Instant
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*

import javax.validation.Valid
import kotlin.collections.ArrayList


@RestController
@RequestMapping("/api")
class FarmPlotController(
        private val farmPlotRepository: FarmPlotRepository, private val accountRepository: AccountRepository,
        private val plantRepository: PlantRepository, private val itemRepository: ItemRepository,
        private val itemAccountRepository: ItemAccountRepository, private val plantAccountRepository: PlantAccountRepository) {


    @GetMapping("/farmPlots")
    fun getFarmPlotById(@RequestHeader("Username") username: String): ResponseEntity<List<FarmPlot>> {
        val farmPlots: List<FarmPlot> = farmPlotRepository.getFarmPlotsByAccount_Username(username = username)
        return ResponseEntity(farmPlots, HttpStatus.OK)
    }

    //TODO find another solution
    @PostMapping("/farmPlots/create")
    fun createNewFarmPlot(@RequestHeader("Username") username: String) {
        val account: Account = accountRepository.findById(username).get()
        for (i in 0..9) {
            val farmPlot = FarmPlot(slotnr = i, account = account)
            farmPlotRepository.save(farmPlot)
        }
    }

    @PutMapping("/farmPlots/plant")
    fun plantFarmPlotByPlantNameAndSlotnr(@RequestHeader("Username") username: String, @Valid @RequestBody farmPlotPlanting: FarmPlotPlanting): ResponseEntity<ArrayList<FarmPlot>> {
        val farmPlots = plantFarmPlot(username, farmPlotPlanting)
        if(farmPlots==null){
            return ResponseEntity(HttpStatus.UNAUTHORIZED)
        }
        return ResponseEntity(farmPlots, HttpStatus.OK)
    }


    @PutMapping("/farmPlots/harvest")
    fun harvestFarmPlotByPlantNameAndSlotnr(
            @RequestHeader("Username") username: String,
            @Valid @RequestBody farmPlotPlanting: FarmPlotPlanting): ResponseEntity<Any> {

        when {
            farmPlotPlanting.plant == "harvest" -> {
                val harvest = harvestPlantsFromFarmPlots(username, farmPlotPlanting)
                if (harvest != null) {
                    addHarvestToInventory(accountRepository.findById(username).get(),harvest)
                    return ResponseEntity(harvest, HttpStatus.OK)
                } else {
                    return ResponseEntity("These plants have not finished growing",HttpStatus.CONFLICT)
                }
            }
            farmPlotPlanting.plant == "remove" -> {
                removePlantsFromFarmPlots(username, farmPlotPlanting)
                return ResponseEntity(HttpStatus.OK)
            }
            else -> {
                return ResponseEntity("Unknown action requested",HttpStatus.BAD_REQUEST)
            }
        }
    }

    //TODO add price for
    @PostMapping("/farmPlots/buyNewFarmPlot")
    fun buyNewFarmPlot(@RequestHeader("Username") username: String) : ResponseEntity<Boolean> {
        val account: Account = accountRepository.findById(username).get()
        val price: Int = when (account.indexOfLastFarmPlotUnlocked) {
            0 -> 500
            1 -> 5_000
            2 -> 50_000
            3 -> 500_000
            else -> return ResponseEntity(HttpStatus.CONFLICT)
        }

        if (account.coinsOwned>= price) {
            val farmPlot = FarmPlot(slotnr = account.indexOfLastFarmPlotUnlocked + 1, account = account)
            farmPlotRepository.save(farmPlot)
            accountRepository.save(account.copy(indexOfLastFarmPlotUnlocked = account.indexOfLastFarmPlotUnlocked + 1, coinsOwned = account.coinsOwned - price))
            return ResponseEntity(true, HttpStatus.OK)
        }
        return ResponseEntity(HttpStatus.CONFLICT)
    }

    @PutMapping("/farmplots/testharvest")
    fun test(){
        val harvest = mutableMapOf<String, Int>()
        harvest["wheat"] = 5
        addHarvestToInventory(accountRepository.findById("sulfax").get(),harvest)
    }


    fun harvestPlantsFromFarmPlots(username: String, farmPlotPlanting: FarmPlotPlanting): MutableMap<String,Int>? {
        val harvest = mutableMapOf<String, Int>().withDefault { 0 }
        farmPlotPlanting.plots.forEach {
            val existingFarmPlot: FarmPlot? = farmPlotRepository.getFarmPlotBySlotnrAndAccount_Username(it, username)
            if(existingFarmPlot==null || (existingFarmPlot.finishedGrowing != null && existingFarmPlot.finishedGrowing > Instant.now().millis)) return null
            val plant: Plant? = existingFarmPlot.plant
            if (plant != null) {
                val amount: Int = (plant.amountWhenFullyGrownMin..plant.amountWhenFullyGrownMax).shuffled().last()
                harvest[plant.name] = harvest.getValue(plant.name) + amount
                val plantAccount:PlantAccount? = plantAccountRepository.getPlantAccountByAccount_UsernameAndPlantName(username,plant.name)
                if(plantAccount==null){
                    plantAccountRepository.save(PlantAccount(account = accountRepository.findById(username).get(),plantName = plant.name,numberOfTimesHarvested = 1, totalAmountHarvested = amount))
                }else{
                    plantAccountRepository.save(plantAccount.copy(numberOfTimesHarvested = plantAccount.numberOfTimesHarvested+1, totalAmountHarvested = plantAccount.totalAmountHarvested+amount))
                }
            }
            val updatedFarmPlot: FarmPlot = existingFarmPlot
                    .copy(plant = null, finishedGrowing = -1)
            farmPlotRepository.save(updatedFarmPlot)
        }
        return harvest
    }

    fun removePlantsFromFarmPlots(username: String, farmPlotPlanting: FarmPlotPlanting) {
        farmPlotPlanting.plots.forEach {
            val existingFarmPlot: FarmPlot = farmPlotRepository.getFarmPlotBySlotnrAndAccount_Username(it, username)
            val updatedFarmPlot: FarmPlot = existingFarmPlot
                    .copy(plant = null, finishedGrowing = -1)
            farmPlotRepository.save(updatedFarmPlot)
        }
    }

    fun plantFarmPlot(username: String, farmPlotPlanting: FarmPlotPlanting): ArrayList<FarmPlot>? {
        val farmPlots: ArrayList<FarmPlot> = ArrayList()
        val plant = plantRepository.findById(farmPlotPlanting.plant).get()

        val plantAccount = plantAccountRepository.getPlantAccountByAccount_UsernameAndPlantName(username,farmPlotPlanting.plant)
        if(plantAccount==null||!plantAccount.unlocked){
            return null
        }
        farmPlotPlanting.plots.forEach {
            val existingFarmPlot: FarmPlot = farmPlotRepository.getFarmPlotBySlotnrAndAccount_Username(it, username)
            val updatedFarmPlot: FarmPlot = existingFarmPlot
                    .copy(plant = plant, finishedGrowing = Instant.now().millis + plant.growthTimeInMillies)
            farmPlotRepository.save(updatedFarmPlot)
            farmPlots.add(updatedFarmPlot)
        }
        return farmPlots
    }

    fun addHarvestToInventory(account: Account, harvest: MutableMap<String,Int>){
        harvest.forEach { plantname, amount ->
            val item:Item? = itemRepository.getItemByName(plantname)
            if(item != null){
                val itemAccount: ItemAccount? = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username = account.username, itemId = item.id)
                if(itemAccount!=null){
                    itemAccountRepository.save(itemAccount.copy(amountOwned = itemAccount.amountOwned+amount))
                }else{
                    val itemAccount2 = ItemAccount(account=account,itemId = item.id,amountOwned=amount)
                    itemAccountRepository.save(itemAccount2)
                }
            }

        }
    }
}