package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.*
import com.merchantsofrome.server.repository.*
// import jdk.jfr.BooleanFlag
// import jdk.jfr.Category
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.util.*
import javax.validation.Valid

@RestController
@RequestMapping("/api/feedback")
class FeedbackController(private val questRepository: QuestRepository, private val feedbackRepository: FeedbackRepository,
                         private val feedbackQuestionRepository: FeedbackQuestionRepository, private val fortuneRepository: FortuneRepository,
                         private val accountRepository: AccountRepository) {

    @GetMapping("/questions")
    fun getFeedbackQuestions(@RequestHeader( "Username") username: String): ResponseEntity<Any> {
        if(questRepository.getQuestByAccount_UsernameAndQuestId(username, 1)?.finished == true &&
                questRepository.getQuestByAccount_UsernameAndQuestId(username, 3)?.finished != true){
            val questions = feedbackQuestionRepository.getFeedbackQuestionsByQuestionSetId(0)
            val list: ArrayList<FeedbackQuestion> = ArrayList()
            questions.forEach{
                if(feedbackRepository.getFeedbackByUsernameAndQuestionId(username,it.id).isEmpty()){
                    list.add(it)
                }
            }
            return ResponseEntity(list, HttpStatus.OK)
        }
        if(questRepository.getQuestByAccount_UsernameAndQuestId(username, 3)?.finished == true){
            val questions = feedbackQuestionRepository.getFeedbackQuestionsByQuestionSetId(1)
            val list: ArrayList<FeedbackQuestion> = ArrayList()
            questions.forEach{
                if(feedbackRepository.getFeedbackByUsernameAndQuestionId(username,it.id).isEmpty()){
                    list.add(it)
                }
            }
            return ResponseEntity(list, HttpStatus.OK)
        }
        return ResponseEntity(emptyList<FeedbackQuestion>(), HttpStatus.OK)
    }

    private fun finishedQuestionSet(username: String, questionSetId: Int): Boolean{
        val questions = feedbackQuestionRepository.getFeedbackQuestionsByQuestionSetId(questionSetId)
        questions.forEach{
            if(feedbackRepository.getFeedbackByUsernameAndQuestionId(username,it.id).isEmpty()){
                return false
            }
        }
        return true
    }

    @PostMapping("/submit")
    fun submitFeedback(@RequestHeader( "Username") username: String, @Valid @RequestBody feedback: Feedback): ResponseEntity<Any> {
        val timestamp = Date()
        val fortune = fortuneRepository.getByUsername(username)
        feedbackRepository.save(feedback.copy(id=0,timestamp = timestamp, fortune = fortune?.fortune))

        val question = feedbackQuestionRepository.findById(feedback.questionId).get()
        if(question.questionType == "end"){
            val a = question.questionText.split("~")[1].split(" ")[0]
            val coins = Integer.parseInt(a)
            val account = accountRepository.getByUsername(username)!!
            accountRepository.save(account.copy(coinsOwned = account.coinsOwned + coins))
        }

        return ResponseEntity(HttpStatus.OK)
    }
}