package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.Account
import com.merchantsofrome.server.model.Category
import com.merchantsofrome.server.model.Item
import com.merchantsofrome.server.model.object_parameters.InventoryObject
import com.merchantsofrome.server.repository.AccountRepository
import com.merchantsofrome.server.repository.ItemAccountRepository
import com.merchantsofrome.server.repository.ItemRepository
// import jdk.jfr.BooleanFlag
// import jdk.jfr.Category
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api")
class ItemController(private val itemAccountRepository: ItemAccountRepository, private val itemRepository: ItemRepository,
                     private val accountRepository: AccountRepository) {

    @GetMapping("/inventory")
    fun getInventory(@RequestHeader( "Username") username: String): ResponseEntity<MutableMap<String,InventoryObject>> {
        val itemAccounts = itemAccountRepository.getAllByAccount_Username(username)
        val inventory: MutableMap<String,InventoryObject> = HashMap()
        inventory["coins"] = InventoryObject("coins",accountRepository.findById(username).get().coinsOwned,Category.undefined,1)

        itemAccounts.forEach {
            val item: Item? = itemRepository.findById(it.itemId).get()
            inventory[item!!.name] = InventoryObject(item.name,it.amountOwned,item.category,item.price)
        }
        return ResponseEntity(inventory, HttpStatus.OK)
    }

    @GetMapping("/items/prices")
    fun getPrices(@RequestHeader( "Username") username: String): ResponseEntity<MutableMap<String,Int>> {
        val items: MutableMap<String,Int> = HashMap()
        itemRepository.findAll().forEach {
            items[it.name] = it.price
        }
        return ResponseEntity(items, HttpStatus.OK)
    }



    @PutMapping("/inventory/sell")
    fun sellItem(@RequestHeader( "Username") username: String, @Valid @RequestBody inventoryObject: InventoryObject ):ResponseEntity<InventoryObject>{
        val item: Item = itemRepository.getItemByName(inventoryObject.name) ?: return ResponseEntity(HttpStatus.BAD_REQUEST)
        val itemAccount = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username,item.id)
        if(itemAccount==null|| itemAccount.amountOwned<inventoryObject.amount){
            return ResponseEntity(HttpStatus.CONFLICT)
        }
        val account:Account = accountRepository.findById(username).get()
        accountRepository.save(account.copy(coinsOwned = account.coinsOwned+inventoryObject.amount*item.price))
        itemAccountRepository.save(itemAccount.copy(amountOwned = itemAccount.amountOwned-inventoryObject.amount))
        return ResponseEntity(InventoryObject(name = "coins", amount = item.price*inventoryObject.amount,category = com.merchantsofrome.server.model.Category.undefined,price = -1),HttpStatus.OK)
    }
}