package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.Plant
import com.merchantsofrome.server.repository.PlantAccountRepository
import com.merchantsofrome.server.repository.PlantRepository
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.util.*
import kotlin.collections.ArrayList

@RestController
@RequestMapping("/api")
class PlantController(private val plantRepository: PlantRepository, private val plantAccountRepository: PlantAccountRepository) {

    @GetMapping("/plants/{name}")
    fun getPlantById(@PathVariable(value = "name") name: String): ResponseEntity<Plant> {
        return plantRepository.findById(name).map { plant ->
            ResponseEntity.ok(plant)
        }.orElse(ResponseEntity.notFound().build())
    }

    @GetMapping("/plants/unlocked")
    fun getUnlockedPlants(@RequestHeader("Username") username: String): ResponseEntity<ArrayList<Plant>> {
        val unlocked : ArrayList<Plant> = ArrayList()
        plantAccountRepository.getPlantAccountByAccount_UsernameAndUnlockedEquals(username,true).forEach {
            unlocked.add(plantRepository.findById(it.plantName).get())
        }
        unlocked.sortWith(Comparator.comparing(Plant::growthTimeInMillies))
        return ResponseEntity(unlocked, HttpStatus.OK)
    }

    @GetMapping("/plants/harvested")
    fun getHarvestedPlants(@RequestHeader("Username") username: String): ResponseEntity<Any>{
        val harvested = HashMap<String, Int>().withDefault { 0 }
        plantAccountRepository.getPlantAccountsByAccount_Username(username).forEach {
            harvested[it.plantName] = it.totalAmountHarvested
        }
        return ResponseEntity(harvested, HttpStatus.OK)
    }
}