package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.*
import com.merchantsofrome.server.model.object_parameters.ProcessData
import com.merchantsofrome.server.repository.*
import org.joda.time.Instant
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api/process")
class ProcessController(private val processRepository: ProcessRepository, private val processAccountRepository: ProcessAccountRepository,
                        private val accountRepository: AccountRepository, private val itemAccountRepository: ItemAccountRepository,
                        private val itemRepository: ItemRepository, private val processItemRepository: ProcessItemRepository) {


    @GetMapping
    fun getProcesses(@RequestHeader("Username") username: String):ResponseEntity<MutableMap<String,ProcessData>>{
        val processes : MutableMap<String, ProcessData> = HashMap()
        val processAccounts: List<ProcessAccount> = processAccountRepository.getAllByAccount_Username(username)
        processAccounts.forEach {
            if(it.active){
                val process = processRepository.findById(it.processId).get()
                processes[process.category]= ProcessData(process.name, it.amount, it.finishTime)
            }
        }
        return ResponseEntity(processes,HttpStatus.OK)
    }


    @PutMapping("/start")
    fun startProcess(@RequestHeader("Username") username: String, @Valid @RequestBody processData: ProcessData): ResponseEntity<HttpStatus> {
        val process: Process = processRepository.getProcessByName(processData.name)
        val processes = processAccountRepository.getAllByAccount_Username(username)

        //Check for active processes
        processes.forEach {
            val currentProcess: Process = processRepository.findById(it.processId).get()
            if (currentProcess.category == process.category && it.active) {
                println("There is aleady an active process in the same category")
                return ResponseEntity(HttpStatus.CONFLICT)
            }
        }

        process.input.forEach {
            val item = itemRepository.findById(it.itemId).get()
            val itemAccount = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username, it.itemId)
            if (itemAccount == null) {
                println("ItemAccount does not exist")
                return ResponseEntity(HttpStatus.CONFLICT)
            }
            println("${item.name}:  Owned: ${itemAccount.amountOwned}, Needed: ${processData.amount * it.amount}")
            if (itemAccount.amountOwned < processData.amount * it.amount) {
                println("Amount of ${item.name} is too low")
                return ResponseEntity(HttpStatus.CONFLICT)
            }
        }


        process.input.forEach {
            val item = itemRepository.findById(it.itemId).get()
            val itemAccount = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username, item.id)
            val updatedItemAccount: ItemAccount = itemAccount!!.copy(amountOwned = itemAccount.amountOwned - processData.amount * it.amount)
            itemAccountRepository.save(updatedItemAccount)
        }
        val processAccount = processAccountRepository.getByAccount_UsernameAndProcessId(username, process.id)
        if(processAccount!=null){
            processAccountRepository.save(processAccount.copy(   finishTime = Instant.now().millis + process.duration*processData.amount, amount = processData.amount, active = true))
        }else {
            processAccountRepository.save(ProcessAccount(processId = process.id, account = accountRepository.findById(username).get(), finishTime = Instant.now().millis + process.duration * processData.amount, amount = processData.amount, active = true))
        }
        return ResponseEntity(HttpStatus.OK)
    }

    @PutMapping("/collect")
    fun collectFromProcess(@RequestHeader("Username") username: String, @Valid @RequestBody processData: ProcessData): ResponseEntity<ProcessData> {
        val process: Process = processRepository.getProcessByName(processData.name)
        val processAccount = processAccountRepository.getByAccount_UsernameAndProcessId(username, process.id)
        if (processAccount == null || !processAccount.active || processAccount.finishTime<=0||processAccount.finishTime > Instant.now().millis) {
            println("There is no process ready")
            return ResponseEntity(HttpStatus.CONFLICT)
        }
        process.output.forEach {
            val itemAccount = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username,it.itemId)
            if(itemAccount!=null) {
                itemAccountRepository.save(itemAccount.copy(amountOwned = itemAccount.amountOwned+processAccount.amount*it.amount))
            }else itemAccountRepository.save(ItemAccount(amountOwned = it.amount*processAccount.amount,itemId = it.itemId,account = accountRepository.findById(username).get()))
        }
        val updatedProcessAccount = processAccount.copy(finishTime = -1,active = false,numberOfTimesCollected = processAccount.numberOfTimesCollected+1, totalAmountCollected = processAccount.totalAmountCollected+processAccount.amount)
        processAccountRepository.save(updatedProcessAccount)
        return ResponseEntity(HttpStatus.OK)
    }
}