package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.*
import com.merchantsofrome.server.repository.*
import org.springframework.web.bind.annotation.*
import org.joda.time.Instant

import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity

@RestController
@RequestMapping("/api/quests")
class QuestController(private val questRepository: QuestRepository, private val plantRepository: PlantRepository,
                      private val farmPlotRepository: FarmPlotRepository, private val accountRepository: AccountRepository,
                      private val plantAccountRepository: PlantAccountRepository, private val itemAccountRepository: ItemAccountRepository,
                      private val itemRepository: ItemRepository, private val processAccountRepository: ProcessAccountRepository,
                      private val processRepository: ProcessRepository, private val mineRepository: MineRepository,
                      private val treePatchRepository: TreePatchRepository, private val treeAccountRepository: TreeAccountRepository) {

    @GetMapping
    fun getAllQuests(@RequestHeader("Username") username: String): List<Quest> = questRepository.getAllByAccount_UsernameAndCheckpointReachedGreaterThan(username, -1)

    @PostMapping("/{questnr}")
    fun startQuest(@RequestHeader("Username") username: String, @PathVariable("questnr") questId: Int, @PathVariable("checkpoint") checkpoint: Int): ResponseEntity<Boolean> {
        if (questId == 1) {
            return ResponseEntity(true, HttpStatus.OK)
        }
        return ResponseEntity(false, HttpStatus.RESET_CONTENT)


    }

    @PutMapping("/{questnr}/{checkpoint}")
    fun doQuestCheckpoint(@RequestHeader("Username") username: String, @PathVariable("questnr") questId: Int, @PathVariable("checkpoint") checkpoint: Int): ResponseEntity<Boolean> {
        val quest = questRepository.getQuestByAccount_UsernameAndQuestId(username, questId)
        if (quest == null || quest.checkpointReached != checkpoint - 1) return ResponseEntity(HttpStatus.CONFLICT)
        val account: Account = accountRepository.findById(username).get()
        when (questId) {
            1 -> when (checkpoint) {
                1 -> {
                    // val account = accountRepository.findById(username).get()
                    accountRepository.save(account.copy(indexOfLastFarmPlotUnlocked = account.indexOfLastFarmPlotUnlocked + 1))
                    farmPlotRepository.save(FarmPlot(slotnr = 0, account = account, plant = plantRepository.findById("wheat").get(), finishedGrowing = Instant.now().millis + 5 * 1000))
                    questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                    return ResponseEntity(HttpStatus.OK)
                }
                2 -> {
                    if (plantAccountRepository.getPlantAccountByAccount_UsernameAndPlantName(username, "wheat")!!.numberOfTimesHarvested >= 0) {
                        val crops = plantAccountRepository.getPlantAccountByAccount_UsernameAndPlantName(username, "wheat")
                        plantAccountRepository.save(crops!!.copy(unlocked = true))
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                3 -> {
                    if (farmPlotRepository.getFarmPlotBySlotnrAndAccount_Username(0, username).plant!!.name == "wheat") {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                4 -> {
                    if (processAccountRepository.getByAccount_UsernameAndProcessId(username, processRepository.getProcessByName("wheatFlour").id)!!.numberOfTimesCollected > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                5 -> {
                    if (processAccountRepository.getByAccount_UsernameAndProcessId(username, processRepository.getProcessByName("flourBread").id)!!.numberOfTimesCollected > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                6 -> {
                    val totalAmountOfBreadMade = processAccountRepository.getByAccount_UsernameAndProcessId(username, processRepository.getProcessByName("flourBread").id)!!.totalAmountCollected
                    val amountOfBreadOwned = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username, itemRepository.getItemByName("bread")!!.id)!!.amountOwned
                    if (totalAmountOfBreadMade > amountOfBreadOwned) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                7 -> {
                    questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1, finished = true))
                    questRepository.save(Quest(questId = 2, account = account, checkpointReached = 0))
                    accountRepository.save(account.copy(coinsOwned = account.coinsOwned+5))
                    return ResponseEntity(HttpStatus.OK)
                }
                else -> return ResponseEntity(HttpStatus.CONFLICT)
            }
            2 -> when (checkpoint) {
                1 -> {
                    if (account.coinsOwned >= 10 || account.indexOfLastMineUnlocked > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                2 -> {
                    if (account.indexOfLastMineUnlocked > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                3 -> {
                    questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                    return ResponseEntity(HttpStatus.OK)
                }
                4 -> {
                    val mines: List<Mine> = mineRepository.getMineByAccount_Username(username)
                    var mined = false
                    mines.forEach {
                        if (it.numberOfTimesMined > 0) {
                            mined = true
                        }
                    }
                    if (mined) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                5 -> {
                    if (processAccountRepository.getByAccount_UsernameAndProcessId(username, processRepository.getProcessByName("smeltIronOre").id)!!.numberOfTimesCollected > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                6 -> {
                    if (processAccountRepository.getByAccount_UsernameAndProcessId(username, processRepository.getProcessByName("ironSword").id)!!.numberOfTimesCollected > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                7 -> {
                    val totalAmountOfSwordsMade = processAccountRepository.getByAccount_UsernameAndProcessId(username, processRepository.getProcessByName("ironSword").id)!!.totalAmountCollected
                    val amountOfSwordsOwned = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username, itemRepository.getItemByName("sword")!!.id)!!.amountOwned
                    if (totalAmountOfSwordsMade > amountOfSwordsOwned) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                8 -> {
                    questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1, finished = true))
                    questRepository.save(Quest(questId = 3, account = account, checkpointReached = 0))
                    return ResponseEntity(HttpStatus.OK)
                }
                else -> return ResponseEntity(HttpStatus.CONFLICT)
            }
            3 -> when (checkpoint) {
                1 -> {
                    if (account.coinsOwned >= 125 || account.indexOfLastTreePatchUnlocked >= 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                2 -> {
                    if (account.indexOfLastTreePatchUnlocked >= 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))

                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                3 -> {
                    val apple: Item = itemRepository.getItemByName("apple")!!
                    val appleItemAccount = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username, apple.id)
                    if(appleItemAccount == null){
                        itemAccountRepository.save(ItemAccount(1,apple.id, account))
                    }else{
                        itemAccountRepository.save(appleItemAccount.copy(amountOwned = appleItemAccount.amountOwned + 1))
                    }
                    questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                    return ResponseEntity(HttpStatus.OK)
                }
                4 -> {
                    if (treePatchRepository.getTreePatchesByAccount_UsernameAndTree_Name(username, "appleTree").size>0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                5 -> {
                    val treeAccount: TreeAccount = treeAccountRepository.getTreeAccountByAccount_UsernameAndTreeName(username, "appleTree")!!
                    val numberOfApples: Int = treeAccount.totalAmountFruitHarvested
                    val numberOfLogs: Int = treeAccount.totalAmountHarvested
                    if (numberOfApples > 0 && numberOfLogs > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                6 -> {
                    if (processAccountRepository.getByAccount_UsernameAndProcessId(username = username, processId = processRepository.getProcessByName("logsPlanks").id)!!.numberOfTimesCollected > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                7 -> {
                    questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1, finished = true))
                    return ResponseEntity(HttpStatus.OK)
                }
                /*5 -> {
                    if (processAccountRepository.getByAccount_UsernameAndProcessId(username = username, processId = processRepository.getProcessByName("ironNails").id)!!.numberOfTimesCollected > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                6 -> {
                    if (itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username = username, itemId = itemRepository.getItemByName("planks")!!.id)!!.amountOwned > 150
                            && itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username = username, itemId = itemRepository.getItemByName("nails")!!.id)!!.amountOwned > 300) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                7 -> {
                    if (processAccountRepository.getByAccount_UsernameAndProcessId(username = username, processId = processRepository.getProcessByName("planksBoat").id)!!.numberOfTimesCollected > 0) {
                        questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                        return ResponseEntity(HttpStatus.OK)
                    }
                    return ResponseEntity(HttpStatus.CONFLICT)
                }
                8 -> {
                    questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1))
                    return ResponseEntity(HttpStatus.OK)
                }
                9 -> {
                    questRepository.save(quest.copy(checkpointReached = quest.checkpointReached + 1, finished = true))
                }*/
            }
        }
        return ResponseEntity(HttpStatus.CONFLICT)
    }
}