package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.*
import com.merchantsofrome.server.repository.*
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import javax.transaction.Transactional
import kotlin.collections.ArrayList

@RestController
@RequestMapping("/api/upgrade")
class UpgradeController(private val plantRepository: PlantRepository, private val plantAccountRepository: PlantAccountRepository,
                        private val itemRepository: ItemRepository, private val accountRepository: AccountRepository,
                        private val treeAccountRepository: TreeAccountRepository, private val treeRepository: TreeRepository,
                        private val itemAccountRepository: ItemAccountRepository) {


    @PutMapping("/seeds/{name}")
    fun unlockSeed(@RequestHeader("Username") username: String, @PathVariable(value = "name") name: String): ResponseEntity<Any> {
        val plants = plantAccountRepository.getPlantAccountByAccount_UsernameAndUnlockedEquals(username, true)
        val harvestingRequirements = when (name) {
            "lettuce" -> {
                var pass = false
                plants.forEach {
                    if (it.plantName == "wheat" && it.totalAmountHarvested >= 15000) pass = true
                }
                pass
            }
            "cucumber" -> {
                var pass = 0
                plants.forEach {
                    if (it.plantName == "lettuce" && it.totalAmountHarvested >= 2000) pass++
                }
                pass == 1
            }
            "barley" -> {
                var pass = 0
                plants.forEach {
                    if (it.plantName == "wheat" && it.totalAmountHarvested >= 30000) pass++
                    if (it.plantName == "cucumber") pass++
                }
                pass == 2
            }
            "carrot" -> {
                var pass = 0
                plants.forEach {
                    if (it.plantName == "barley") pass++
                    if (it.plantName == "lettuce" && it.totalAmountHarvested >= 3000) pass++
                    if (it.plantName == "cucumber" && it.totalAmountHarvested >= 1000) pass++
                }
                pass == 3
            }
            else -> return ResponseEntity("Cant find a seed to unlock", HttpStatus.NOT_FOUND)
        }
        if (!harvestingRequirements) {
            return ResponseEntity("Harvesting requirements not met", HttpStatus.CONFLICT)
        }

        val account: Account = accountRepository.findById(username).get()
        val price: Int = when (name) {
            "lettuce" -> 300
            "cucumber" -> 500
            "barley" -> 2000
            "carrot" -> 3000
            else -> return ResponseEntity("Cant find a seed to unlock", HttpStatus.NOT_FOUND)
        }
        if (account.coinsOwned < price) {
            return ResponseEntity("You don't have enough money", HttpStatus.CONFLICT)
        }

        val itemAccounts = itemAccountRepository.getAllByAccount_Username(username)
        val plantPriceRequirements = when (name) {
            "lettuce" -> true
            "cucumber" -> {
                var pass = false
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("lettuce")!!.id && it.amountOwned >= 500) pass = true
                }
                pass
            }
            "barley" -> {
                var pass = false
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("wheat")!!.id && it.amountOwned >= 10000) pass = true
                }
                pass
            }
            "carrot" -> {
                var pass = 0
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("lettuce")!!.id && it.amountOwned >= 1000) pass++
                    if (it.itemId == itemRepository.getItemByName("cucumber")!!.id && it.amountOwned >= 500) pass++
                }
                pass == 2
            }
            else -> return ResponseEntity("Cant find a seed to unlock 2", HttpStatus.NOT_FOUND)
        }
        if (!plantPriceRequirements) {
            return ResponseEntity("You don't have enough $name", HttpStatus.CONFLICT)
        }
        when (name) {
            "lettuce" -> {}
            "cucumber" -> {
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("lettuce")!!.id){
                        itemAccountRepository.save(it - 500)
                    }
                }
            }
            "barley" -> {
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("wheat")!!.id){
                        itemAccountRepository.save(it - 10000)
                    }
                }
            }
            "carrot" -> {
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("lettuce")!!.id){
                        itemAccountRepository.save(it - 1000)
                    }
                    if (it.itemId == itemRepository.getItemByName("cucumber")!!.id){
                        itemAccountRepository.save(it - 500)
                    }
                }
            }
            else -> return ResponseEntity("Cant find a seed to unlock 3", HttpStatus.NOT_FOUND)
        }
        plantAccountRepository.save(PlantAccount(plantName = name, account = account, unlocked = true))
        accountRepository.save(account.copy(coinsOwned = account.coinsOwned - price))

        return ResponseEntity(HttpStatus.OK)
    }

    @PutMapping("/fruits/{name}")
    fun unlockFruit(@RequestHeader("Username") username: String, @PathVariable(value = "name") name: String): ResponseEntity<Any> {
        val treeAccounts = treeAccountRepository.getTreeAccountsByAccount_Username(username)
        val harvestingRequirements = when (name) {
            "apple" -> true
            "peach" -> {
                var pass = false
                treeAccounts.forEach {
                    if (itemRepository.findById(it.tree!!.fruitId).get().name == "apple" && it.totalAmountFruitHarvested >= 250) pass = true
                }
                pass
            }
            "orange" -> {
                var pass = 0
                treeAccounts.forEach {
                    if (itemRepository.findById(it.tree!!.fruitId).get().name == "apple" && it.totalAmountFruitHarvested >= 500) pass++
                    if (itemRepository.findById(it.tree.fruitId).get().name == "peach" && it.totalAmountFruitHarvested >= 250) pass++
                }
                pass == 2
            }
            "chestnut" -> {
                var pass = 0
                treeAccounts.forEach {
                    if (itemRepository.findById(it.tree!!.fruitId).get().name == "peach" && it.totalAmountFruitHarvested >= 500) pass++
                    if (itemRepository.findById(it.tree.fruitId).get().name == "orange" && it.totalAmountFruitHarvested >= 2000) pass++
                }
                pass == 2
            }
            else -> return ResponseEntity("Cant find a seed to unlock", HttpStatus.NOT_FOUND)
        }
        if (!harvestingRequirements) {
            return ResponseEntity("Harvesting requirements not met", HttpStatus.CONFLICT)
        }
        val account: Account = accountRepository.findById(username).get()
        val price: Int = when (name) {
            "apple" -> 150
            "peach" -> 1000
            "orange" -> 2000
            "chestnut" -> 3000

            else -> return ResponseEntity("Cant find a seed to unlock", HttpStatus.NOT_FOUND)
        }
        if (account.coinsOwned < price) {
            return ResponseEntity("You don't have enough money", HttpStatus.CONFLICT)
        }
        val itemAccounts = itemAccountRepository.getAllByAccount_Username(username)
        val fruitPriceRequirements = when (name) {
            "apple" -> true
            "peach" -> true
            "orange" -> {
                var pass = 0
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("apple")!!.id && it.amountOwned >= 200) pass++
                    if (it.itemId == itemRepository.getItemByName("peach")!!.id && it.amountOwned >= 100) pass++
                }
                pass == 2
            }
            "chestnut" -> {
                var pass = 0
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("peach")!!.id && it.amountOwned >= 200) pass++
                    if (it.itemId == itemRepository.getItemByName("orange")!!.id && it.amountOwned >= 500) pass++
                }
                pass == 2
            }
            else -> return ResponseEntity("Cant find a seed to unlock 2", HttpStatus.NOT_FOUND)
        }
        if (!fruitPriceRequirements) {
            return ResponseEntity("You don't have enough $name", HttpStatus.CONFLICT)
        }
        when (name) {
            "apple" -> {}
            "peach" -> {}
            "orange" -> {
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("apple")!!.id){
                        itemAccountRepository.save(it - 200)
                    }
                    if (it.itemId == itemRepository.getItemByName("peach")!!.id){
                        itemAccountRepository.save(it - 100)
                    }
                }
            }
            "chestnut" -> {
                itemAccounts.forEach {
                    if (it.itemId == itemRepository.getItemByName("peach")!!.id){
                        itemAccountRepository.save(it - 200)
                    }
                    if (it.itemId == itemRepository.getItemByName("orange")!!.id){
                        itemAccountRepository.save(it - 500)
                    }
                }
            }
            else -> return ResponseEntity("Cant find a seed to unlock 3", HttpStatus.NOT_FOUND)
        }
        val item = itemRepository.getItemByName(name)!!
        val oldItemAccount = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username, item.id)
        itemAccountRepository.save(oldItemAccount?.copy(amountOwned = 1)
                ?: ItemAccount(itemId = item.id, amountOwned = 1, account = account))
        accountRepository.save(account.copy(coinsOwned = account.coinsOwned - price))
        val oldTreeAccount = treeAccountRepository.getTreeAccountByAccount_UsernameAndTreeName(account.username,name+"Tree")
        if(oldTreeAccount==null){
            treeAccountRepository.save(TreeAccount(totalAmountHarvested = 0, totalAmountFruitHarvested = 0, account = account, numberOfTimesFruitHarvested = 0, numberOfTimesHarvested = 0, tree = treeRepository.getTreeByName(name+"Tree")))
        }
        return ResponseEntity(HttpStatus.OK)
    }
}