package com.merchantsofrome.server.model


import com.fasterxml.jackson.annotation.JsonIgnore
import com.fasterxml.jackson.annotation.JsonManagedReference
import com.fasterxml.jackson.annotation.JsonProperty
import javax.persistence.*
import javax.validation.constraints.NotNull
import javax.validation.constraints.Size


@Entity
@Table(name="Account")
data class Account (
        @Id
        @Column(name = "username", unique = true, nullable = false)
        @NotNull
        @Size(min = 3, max = 32)
        var username: String = "",

        val alias: String? = null,

        @Size(min = 8)
        @get:JsonIgnore
        @set:JsonProperty
        var password: String = "",

        @get: JsonIgnore
        val salt: String = "",

        @get: JsonIgnore
        val lastPlayedVersion: Int = 0,

        internal val coinsOwned: Int = 0,

        @OneToMany(mappedBy = "account")
        @JsonManagedReference
        val items: MutableSet<ItemAccount> = HashSet(),

        @OneToMany
        val processes: MutableSet<ProcessAccount> = HashSet(),

        //Farm
        val indexOfLastFarmPlotUnlocked: Int = -1,

        @OneToMany(mappedBy = "account")
        @JsonManagedReference
        val crops: MutableSet<PlantAccount> = HashSet(),

        @OneToMany(mappedBy = "account")
        @JsonManagedReference
        val farmPlots: MutableSet<FarmPlot> = HashSet(),

        @OneToMany(mappedBy = "account")
        @JsonManagedReference
        val quests: MutableSet<Quest> = HashSet(),

        // Mines
        val indexOfLastMineUnlocked: Int = 0,

        @OneToMany(mappedBy = "account")
        @JsonManagedReference
        val mines: MutableSet<Mine> = HashSet(),

        // Woodcutting
        val indexOfLastTreePatchUnlocked: Int = -1,

        @OneToMany(mappedBy = "account")
        @JsonManagedReference
        val treePatches: MutableSet<TreePatch> = HashSet(),

        val rulesAccepted: Boolean = false
){
       constructor(username: String) : this(username=username, salt = "123asdfghjklqwertyuiozxcvbnm")
}

