package com.merchantsofrome.server.model

import com.fasterxml.jackson.annotation.JsonBackReference
import com.merchantsofrome.server.model.id_class.ItemAccountPK
import java.lang.IllegalArgumentException
import javax.persistence.*

@Entity(name = "ItemAccount")
@IdClass(ItemAccountPK::class)
data class ItemAccount(
        val amountOwned: Int = 0,

        @Id
        var itemId: Int = -1,

        @Id
        @ManyToOne(fetch = FetchType.LAZY)
        @JsonBackReference
        var account: Account?=null


){
        operator fun plus(other: ItemAccount): ItemAccount{
                if(this.itemId != other.itemId){
                        throw IllegalArgumentException("Different items")
                }
                return this.copy(amountOwned = this.amountOwned + other.amountOwned)
        }

        operator fun minus(other: ItemAccount): ItemAccount{
                return this + -other
        }

        operator fun unaryMinus(): ItemAccount{
                return this.copy(amountOwned = -this.amountOwned)
        }

        operator fun unaryPlus(): ItemAccount{
                return this.copy(amountOwned = +this.amountOwned)
        }

        operator fun inc(): ItemAccount{
                return this.copy(amountOwned = this.amountOwned+1)
        }

        operator fun dec(): ItemAccount{
                return this.copy(amountOwned = this.amountOwned-1)
        }

        operator fun plus(amount: Int): ItemAccount{
                return this.copy(amountOwned = this.amountOwned + amount)
        }

        operator fun minus(amount: Int): ItemAccount{
                return this + -amount
        }

        operator fun compareTo(amount: Int): Int{
               return this.amountOwned - amount
        }
}