import React, {Component} from 'react';
import './App.css';
import Game from "./Game";
import LoginPage from "./login/LoginPage";
import {connect} from "react-redux";
import {DialogueStandards} from "./dialogue/DialogueStandards";
import {Fetch, Strings} from "./utils";
import Div100vh from 'react-div-100vh';

enum Dialogue{
    None = "None",
    Unknown = "Unknown",
    Unauthorized = "Unauthorized",
    Conflict = "Conflict",
}
interface IAppState{
    dialogue: Dialogue
    errorMessage: string
}

interface IAppProps {
    loggedIn: boolean
}

const mapStateToProps = (state: any) => {
    return {
        loggedIn: state.loggedIn
    }
};

function mapDispatchToProps(dispatch: any) {
    return {

    }
}

class ConnectedApp extends Component<IAppProps, IAppState> {

    state = {
        dialogue: Dialogue.None,
        errorMessage: "Unspecified error"
    };

    constructor(props: IAppProps) {
        super(props);
        this.showError = this.showError.bind(this);
        this.closeDialogue = this.closeDialogue.bind(this);
    }

    componentDidMount() {
        Fetch.showError = this.showError
    }

    showError(response: Response){
        this.setState({
            dialogue: (response.statusText in Dialogue)?response.statusText as unknown as Dialogue:Dialogue.Unknown
        });
        let resClone = response.clone();
        response.text().then((errorMessage)=>{
            if(Strings.isJsonString(errorMessage)){
                resClone.json().then(json=>{
                    this.setState({
                        errorMessage: `${json.error} -> ${json.message}`
                    })
                });
                return;
            }
            this.setState({
                errorMessage
            })
        })
    }


    showDialogue(){
        switch(this.state.dialogue){
            case Dialogue.None:
                return null;
            case Dialogue.Unknown:
                return this.unknownDialogue();
            case Dialogue.Conflict:
                return this.desyncDialogue();
            case Dialogue.Unauthorized:
                return this.unauthorizedDialogue();
            default:
                return this.createErrorDialogue(this.state.dialogue, "Something went very wrong")
        }
    }

    createErrorDialogue(errorType: string, description: string, onClickConfirm: ()=>void = ()=>location.reload(), refresh: boolean = true){
        return DialogueStandards.CONFIRM_DIALOGUE_ELEMENT_BODY(
            `${errorType} error`,
            <div>
                {description}
                <br/>
                {refresh?"Please refresh to continue playing.":""}
                <br/>
                <br/>
                Error message: {this.state.errorMessage}
            </div>,
            onClickConfirm
        )
    }

    unknownDialogue(){
        return this.createErrorDialogue("Unknown", "Something went wrong.");
    }

    desyncDialogue(){
        return this.createErrorDialogue("Desynchronization", "The client has been desynchronized from the server.");
    }

    unauthorizedDialogue(){
        let onClick = undefined;
        let refresh = true;
        if(this.state.errorMessage === "Wrong combination of Username and Password"){
            onClick = this.closeDialogue;
            refresh=false
        }
        return this.createErrorDialogue("Authorization", "Your user credentials are wrong or outdated.",onClick,refresh);
    }

    closeDialogue(){
        this.setState({dialogue: Dialogue.None})
    }


    render() {
        return (
            <Div100vh className="App">
                <div className={"dialogue-app"}>{this.showDialogue()}</div>
                {this.props.loggedIn ? <Game /> : <LoginPage/>}
            </Div100vh>
        )
    }

}

const App = connect(mapStateToProps, mapDispatchToProps)(ConnectedApp);

export default App;