import React, { Component } from 'react';
import './Inventory.css';
import {Fetch, InventoryList, InventoryTools, ItemFactory, ProcessDataList, Strings} from './utils'
import {ItemBox} from "./ItemBox";
import {Tab} from "./Enum";
import {connect} from "react-redux";
import {increaseInventory} from "./actions/actions";
import SubInventory from "./SubInventory";
import {Item} from "./Items/Item";
import {Items, ItemsName} from "./Items/Items";
import {DialogueStandards} from "./dialogue/DialogueStandards";

enum Dialogue {
    None,
    Sell,
    Error
}

interface IInventoryState{
    dialogue: Dialogue,
    error: string,
    item: Item
}

interface IInventoryProps{
    inventory: InventoryList,
    currentTab: Tab,
    increaseInventory: (item: Item) => void
}

const mapStateToProps = (state: any) => {
    return {
        currentTab: state.currentTab,
        inventory: state.inventory
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        increaseInventory: (item: Item) => dispatch(increaseInventory(item))
    }
}


class ConnectedInventory extends Component<IInventoryProps,IInventoryState>{
    state = {
        dialogue: Dialogue.None,
        error: "Unspecified error",
        item: new Items(ItemsName.empty)
    };

    constructor(props: IInventoryProps) {
        super(props);
        // this.onClick = this.onClick.bind(this);

    }

    componentDidMount() {
    }

    /*sellToArmyDialogue() {
        let name = this.state.item.name;
        let list: ProcessDataList = {
        };

        return DialogueStandards.INPUT_DIALOGUE(
            "Army",
            `How many ${name} do you wish to sell to the army?`,
            this.state.item.amount,
            (value: number) => {
                let item: Item = Object.assign({}, this.state.item);
                item.amount = value;
                if (value > 0 && this.checkItem(name, value)) {
                    Fetch.put("inventory/sell", item)
                        .then(response => response.json())
                        .then((data: Item) => {
                            let outputName = data.name;
                            let amount = data.amount;
                            this.removeItem(name, value);
                            let item = ItemFactory.create(outputName, amount);
                            this.props.increaseInventory(item);
                            this.closeDialogue();
                        })
                        .catch(error => {
                            console.log(`Error: ${error}`);
                            this.setState({
                                dialogue: Dialogue.Error,
                                error: `You don't have ${value} ${name}`
                            });

                        });
                    this.closeDialogue();
                } else {
                    this.setState({
                        dialogue: Dialogue.Error,
                        error: `You dont have ${value} ${name}`
                    });
                }
            },
            () => {this.closeDialogue()});
    }

    checkItem(name: string, amount:number = 1): boolean{
        return InventoryTools.checkItem(name, amount, this.props.inventory)
    }

    removeItem(name: string, amount: number){
        InventoryTools.removeItem(name, amount, this.props.increaseInventory)
    }

    errorDialogue() {
        return DialogueStandards.CONFIRM_DIALOGUE(
            Strings.decamelize(this.state.item.name),
            this.state.error,
            () => this.closeDialogue()
        );
    }

    showDialogue() {
        switch (this.state.dialogue) {
            case Dialogue.None:
                return null;
            case Dialogue.Sell:
                if (this.props.currentTab == Tab.inventory) {
                    return this.sellToArmyDialogue();
                }
                return this.closeDialogue();
            case Dialogue.Error:
                if (this.props.currentTab == Tab.inventory) {
                    return this.errorDialogue();
                }
                return this.closeDialogue();
        }
    }

    closeDialogue() {
        this.setState({
            dialogue: Dialogue.None
        });
    }

    onClick(item: Item) {
        this.setState({
            item: item,
            dialogue: Dialogue.Sell
        })
    }*/

    render() {
        return (
            <div className="Inventory">
                {Object.keys(this.props.inventory)
                    .filter(key => this.props.inventory[key].amount > 0)
                    .filter(key => this.props.inventory[key].displayInTab(this.props.currentTab))
                    .map((key,i) => <ItemBox item={this.props.inventory[key]} key={i} sell={() => {}}/>)}
            <br/> <br/>
            </div>
        )
    }
}

const Inventory = connect(mapStateToProps, mapDispatchToProps)(ConnectedInventory);

export default Inventory;
