import React, { Component } from 'react';
import './Dialogue.css';
import Div100vh from 'react-div-100vh'

interface IDialogueState{
    landscape: boolean,
}

interface IDialogeProps{
    title: string,
    close?: ()=>void,
    showFade?: boolean
}

class Dialogue extends Component<IDialogeProps,IDialogueState>{
    state = {
        landscape: window.matchMedia('(orientation: landscape)').matches
    };


    private timeout: any;

    constructor(props: IDialogeProps) {
        super(props);
        this.onResize = this.onResize.bind(this);
        this.listener = this.listener.bind(this);
    }

    onResize(evt: Event){
        this.setState({
            landscape: window.matchMedia('(orientation: landscape)').matches
        })
    };

    listener(evt: Event){
        clearTimeout(this.timeout);

        this.timeout = setTimeout(() => {
            this.timeout = null;
            this.onResize(evt);
        }, 16);
    }

    componentDidMount() {
        window.addEventListener("resize", this.listener)
    }

    componentWillUnmount(): void {
        clearTimeout(this.timeout);
        window.removeEventListener("resize", this.listener)
    }

    getHeight(){
        return this.state.landscape? "100rvh" : "calc(100rvh - 70px)";
    }

    render() {
        return (
            <Div100vh style={{height: this.getHeight()}} className={`dialogue-wrapper ${this.props.showFade === false?"":"fade"}`} onClick={this.props.close?this.props.close:()=>{}}>
                <div className={`dialogue ${this.props.showFade === false?"":"top-dialogue"}`} onClick={(event)=>event.stopPropagation()}>
                    <div className="dialogue-title">
                        {this.props.title}
                    </div>
                    <div className="dialogue-content">
                        {this.props.children}
                    </div>
                </div>
            </Div100vh>
        )
    }
}

export default Dialogue;
