import React, { Component } from 'react';
import './DialogueNumberInput.css';
import Button from "./Button";

interface IDialogueNumberInputState{
    value: string
}

interface IDialogueNumberInputProps{
    onClick: (value: number)=>void
    onCancel: ()=>void
    onChange: (value: number)=>void
    initialValue: number
    maxValue: number
}

class DialogueNumberInput extends Component<IDialogueNumberInputProps,IDialogueNumberInputState>{


    constructor(props: IDialogueNumberInputProps) {
        super(props);
        this.onChange = this.onChange.bind(this);
        let initialValue = Math.min(this.props.initialValue, this.props.maxValue);
        this.state = {
            value: ""+initialValue
        };
        this.change = this.change.bind(this);
        this.set = this.set.bind(this);
        this.clear = this.clear.bind(this);
    }

    componentDidMount() {
    }

    onChange(event: any){
        let value = event.target.value;
        this.set(value);
    }

    change(value: number){
        this.set(Number(this.state.value)+value)
    }

    set(value: number){
        value = Math.min(value, this.props.maxValue);
        value = Math.max(0,value);
        this.props.onChange(value);
        this.setState({
            value: ""+value
        });
    }

    clear(){
        this.setState({
            value: ""
        })
    }


    render() {
        return (
            <div className="dialogue-number-input">
                <div className="dialogue-number-input-div">
                    <Button onClick={()=>this.change(-1)} text={"-"}/>
                    <input
                        className="dialogue-number-input-input"
                        type={"number"}
                        value={this.state.value}
                        onChange={this.onChange}
                        onClick={this.clear}
                        min={1}
                        max={this.props.maxValue}/>
                    <Button onClick={()=>this.change(1)} text={"+"}/>
                    <Button onClick={()=>this.set(this.props.maxValue)} text={"Max"} />
                </div>
                <div className="dialogue-number-input-buttons">
                    <Button onClick={()=>this.props.onClick(Number(this.state.value))} text={"Confirm"}/>
                    <Button onClick={this.props.onCancel} text={"Cancel"}/>
                </div>
            </div>
        )
    }
}

export default DialogueNumberInput;
