import DialogueBody from ".//DialogueBody";
import React from "react";
import Dialogue from "./Dialogue";
import Button from "./Button";
import DialogueFooter from "./DialogueFooter";
import {on} from "cluster";
import DialogueNumberInput from "./DialogueNumberInput";
import DialogueTextInput from "./DialogueTextInput";


export class DialogueStandards{
    static YES_NO_DIALOGUE(
        title: string,
        body: string,
        onClickYes: ()=>void,
        onClickNo: ()=>void
    ){
        //console.log(title, body);
        return (
            <Dialogue title={title} close={onClickNo}>
                <DialogueBody>
                    <div>{body}</div>
                </DialogueBody>
                <DialogueFooter>
                    <Button text={"Yes"} onClick={onClickYes}/>
                    <Button text={"No"} onClick={onClickNo}/>
                </DialogueFooter>
            </Dialogue>
        )
    }

    static YES_NO_DIALOGUE_ELEMENT_BODY(
        title: string,
        body: JSX.Element,
        onClickYes: () => void,
        onClickNo: () => void
    ) {
        return (
            <Dialogue title={title} close={onClickNo}>
                <DialogueBody>
                    {body}
                </DialogueBody>
                <DialogueFooter>
                    <Button text={"Yes"} onClick={onClickYes}/>
                    <Button text={"No"} onClick={onClickNo}/>
                </DialogueFooter>
            </Dialogue>
        )
    }

    static CONFIRM_DIALOGUE(
        title: string,
        body: string,
        onClickConfirm: ()=>void,
        showFade: boolean = true
    ){
        return this.CONFIRM_DIALOGUE_ELEMENT_BODY(title, <div>{body}</div>, onClickConfirm, showFade)
    }

    static CONFIRM_DIALOGUE_ELEMENT_BODY(
        title: string,
        body: JSX.Element,
        onClickConfirm: ()=>void,
        showFade: boolean = true
    ){
        //console.log(title, body);
        return (
            <Dialogue title={title} close={showFade?onClickConfirm:()=>{}} showFade={showFade}>
                <DialogueBody>
                    {body}
                </DialogueBody>
                <DialogueFooter>
                    <Button text={"Confirm"} onClick={onClickConfirm}/>
                </DialogueFooter>
            </Dialogue>
        )
    }
    static CANCEL_DIALOGUE_ELEMENT_BODY(
        title: string,
        body: JSX.Element,
        onClickCancel: ()=>void,
    ){
        //console.log(title, body);
        return (
            <Dialogue title={title} close={onClickCancel}>
                <DialogueBody>
                    {body}
                </DialogueBody>
                <DialogueFooter>
                    <Button text={"Cancel"} onClick={onClickCancel}/>
                </DialogueFooter>
            </Dialogue>
        )
    }

    static INPUT_DIALOGUE(
        title: string,
        body: string,
        onClickConfirm: (value: number) => void,
        onClickCancel: () => void,
        onChange: (value: number) => void = (value: number) => {},
        initialValue: number = 1,
        maxValue: number = Number.MAX_SAFE_INTEGER,

    ){
        return this.INPUT_DIALOGUE_ELEMENT_BODY(
            title,
            <div>{body}</div>,
            onClickConfirm,
            onClickCancel,
            onChange,
            initialValue,
            maxValue)
    }

    static INPUT_DIALOGUE_ELEMENT_BODY(
        title: string,
        body: JSX.Element,
        onClickConfirm: (value: number) => void,
        onClickCancel: () => void,
        onChange: (value: number) => void = (value: number) => {},
        initialValue: number = 1,
        maxValue: number = Number.MAX_SAFE_INTEGER,

    ){
        return (
            <Dialogue title={title} close={onClickCancel}>
                <DialogueBody>
                    {body}
                </DialogueBody>
                <DialogueFooter>
                    <DialogueNumberInput
                        onClick={onClickConfirm}
                        onCancel={onClickCancel}
                        onChange= {onChange}
                        initialValue={initialValue}
                        maxValue={maxValue}/>
                </DialogueFooter>
            </Dialogue>
        )
    }

    static TEXT_INPUT_DIALOGUE(
        title: string,
        body: string,
        onClickConfirm: (value: string) => void,
        onClickCancel: () => void = ()=>{},
        onChange: (value: string) => void = (value: string) => {},
        maxCharacters: number = 1000,
        showFade: boolean = true
    ){
        return this.TEXT_INPUT_DIALOGUE_ELEMENT_BODY(
            title,
            <div>{body}</div>,
            onClickConfirm,
            onClickCancel,
            onChange,
            maxCharacters,
            showFade
    )
    }

    static TEXT_INPUT_DIALOGUE_ELEMENT_BODY(
        title: string,
        body: JSX.Element,
        onClickConfirm: (value: string) => void,
        onClickCancel: () => void = ()=>{},
        onChange: (value: string) => void = (value: string) => {},
        maxCharacters: number = 1000,
        showFade: boolean = true
    ){
        return (
            <Dialogue title={title} close={showFade?onClickCancel:()=>{}} showFade={showFade}>
                <DialogueBody>
                    <DialogueTextInput
                        maxCharacters={maxCharacters}
                        onClick={onClickConfirm}
                        onChange= {onChange}>
                        {body}
                    </DialogueTextInput>
                </DialogueBody>
            </Dialogue>
        )
    }
}
