import React, {Component} from 'react';
import {Plant} from "./Plant";
import {Time} from "../utils";
import './FarmingPlot.css';
import {Item} from "../Items/Item";
import {CropName} from "../Items/Crops";
import {increaseInventory, plantSeeds, uprootSeed} from "../actions/actions";
import {connect} from "react-redux";

interface IFarmingPlotProps {
    plant: Plant,
    chosen: boolean,
    harvest: (plant: Plant, finished: boolean, timeLeft: number) => void,
    increaseInventory: (item: Item) => void,
    plantSeeds: (plants: Plant[]) => void,
}

interface IFarmingPlotState {
    timeLeft: number;
}

function mapDispatchToProps(dispatch: any) {
    return {
        increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
        plantSeeds: (plants: Plant[]) => dispatch(plantSeeds(plants)),

    };
}

class ConnectedFarmingPlot extends Component<IFarmingPlotProps,IFarmingPlotState> {
    private interval: any;
    state = {
        timeLeft: 1e9,
    };

    constructor(props: IFarmingPlotProps) {
        super(props);
        this.harvest = this.harvest.bind(this);
    }

    startInterval(){
        clearInterval(this.interval);
        this.interval = setInterval(() => this.tick(), 1000);
    }

    componentDidMount() {
        if(this.props.plant.timeLeft()>0){
            this.startInterval();
        }
        //fetch time
        this.setState({
            timeLeft: this.props.plant.timeLeft()
        });
    }

    componentDidUpdate(prevProps: IFarmingPlotProps){
        //console.log("update", this.state, prevProps);
        if(prevProps.plant.name != this.props.plant.name){
            if(!Time.hasHappened(this.props.plant.finishTime)){
                this.startInterval();
                this.setState({
                    timeLeft: this.props.plant.timeLeft(),
                })
            }
            this.setState({
                timeLeft: this.props.plant.timeLeft()
            })
        }
    }

    harvest(){
        this.props.harvest(this.props.plant, this.state.timeLeft<=0, this.state.timeLeft);
    }

    tick(){
        if(this.props.plant.name=="empty" || this.state.timeLeft <=0){
            clearInterval(this.interval);
            return;
        }
        this.setState(state => ({
            timeLeft: state.timeLeft-1
        }));
    }

    componentWillUnmount() {
        clearInterval(this.interval);
    }

    render() {
        return (
        <div className="farming-plot">
            <div className="farming-plot-title">
                <h4 className="farming-plot-title-name">{this.props.plant.Name}</h4>
                <h5 className="farming-plot-title-timer">{Time.timeFormat(this.state.timeLeft)}</h5>
            </div>
            <div className={`farming-plot-body${this.props.chosen?" selected":""}`} onClick={this.harvest}>
                <img className="farming-plot-body-img" src={this.props.plant.imageUrl(this.state.timeLeft)}
                     alt={this.props.plant.name}/>
            </div>
        </div>
        )
    }
}

const FarmingPlot = connect(null, mapDispatchToProps)(ConnectedFarmingPlot);

export default FarmingPlot;
