import {Crop, CropName} from "../Items/Crops";

abstract class PlantDatabase{
    static data: any = {
        empty: {
            timeToGrow: -1,
            stages: 1
        },
        wheat: {
            timeToGrow: 60,
            stages: 3
        },
        grape: {
            timeToGrow: 20,
            stages: 3
        },
        lettuce: {
            timeToGrow: 3600,
            stages: 3
        },
        cucumber: {
            timeToGrow: 6*3600,
            stages: 3
        },
        barley: {
            timeToGrow: 300,
            stages: 3
        },
        carrot: {
            timeToGrow: 3*3600,
            stages: 3
        }
    };
}

export interface PlantData{
    name: CropName,
    timeLeft: number
}

export class Plant{
    name: CropName;
    Name: string;
    timeToGrow: number;
    stages: number;
    finishTime: Date;
    plotId: number;

    constructor(name: CropName, plotId: number = -1, finishTime?: Date) {
        let plantData = PlantDatabase.data[name];
        this.timeToGrow = plantData.timeToGrow;
        this.stages = plantData.stages;
        this.name = name;
        this.Name = name.substr(0,1).toUpperCase()+name.substr(1);
        this.plotId = plotId;
        if(finishTime){
            this.finishTime = finishTime
        }else{
            let finishDate = new Date();
            finishDate.setSeconds(finishDate.getSeconds()+this.timeToGrow);
            this.finishTime = finishDate;
        }

    }

    timeLeft(){
        return Math.max(0,Math.round((this.finishTime.getTime()-new Date().getTime())/1000));
    }

    imageUrl(time: number){
        return `images/farming/${this.name}${this.stage(time)}.png`
    }

    imageUrlSeed(){
        return `images/${this.name}Seeds.png`
    }

    stage(time: number){
        return Math.min(this.stages-Math.ceil(time/(this.timeToGrow/(this.stages-1))),this.stages);
    }

    finished(time: number){
        return time>this.timeToGrow;
    }

    produce(){
        return new Crop(this.name)
    }
}