import React, { Component } from "react";
import LoginPage from "./LoginPage";
import {Fetch} from "../utils";
import Button from "../dialogue/Button";
import "./CreateAccountPage.css";

interface ICreateAccountPageProps {
    // loggedInState: any;
}

interface ICreateAccountPageState {
    goBack: boolean;
    usernameNewAccount: string;
    passwordNewAccount: string;
    confirmPassword: string;
    hiddenText: string;
    hiddenTextError: string;
}

class CreateAccountPage extends Component<ICreateAccountPageProps, ICreateAccountPageState> {

    state = {
        goBack: false,
        usernameNewAccount: "",
        passwordNewAccount: "",
        confirmPassword: "",
        hiddenText: "",
        hiddenTextError: ""
    };

    constructor(props: ICreateAccountPageProps) {
        super(props);
        this.backToLoginPageClick = this.backToLoginPageClick.bind(this);
        this.usernameChange = this.usernameChange.bind(this);
        this.passwordChange = this.passwordChange.bind(this);
        this.confirmPasswordChange = this.confirmPasswordChange.bind(this);
        this.addUserClick = this.addUserClick.bind(this);
    }

    backToLoginPageClick() {
        this.setState({
            goBack: true
        })
    }

    usernameChange(event: any) {
        let username = event.target.value;
        this.setState({
            usernameNewAccount: username.trim()
        });
    }

    passwordChange(event: any) {
        let password = event.target.value;
        this.setState({
            passwordNewAccount: password
        });
    }

    confirmPasswordChange(event: any) {
        let confirmPassword = event.target.value;
        this.setState({
            confirmPassword: confirmPassword
        })
    }

    addUserClick(event: any) {
        let username = this.state.usernameNewAccount;
        let password = this.state.passwordNewAccount;
        let confirmPassword = this.state.confirmPassword;

        if (username != null && username != "" && password != null && password != "" && confirmPassword != null && confirmPassword != "") {
            if (password === confirmPassword) {

                // TODO: Hash passord
                let account = {username: username, password: password};
                Fetch.post("/api/createNewAccount", /*account*/ account).then(response => {
                    if (response.status === 200) {
                        this.setState({
                            hiddenText: "Your user has been created",
                            hiddenTextError: ""
                        });
                        setInterval(this.backToLoginPageClick, 1000);
                    } else {
                        this.setState({
                            hiddenTextError: response.status + " " + response.statusText
                        });
                    }
                });
            } else {
                this.setState({
                    usernameNewAccount: "",
                    passwordNewAccount: "",
                    confirmPassword: "",
                    hiddenTextError: "Passwords don't match"
                });
            }
        } else {
            this.setState({
                usernameNewAccount: "",
                passwordNewAccount: "",
                confirmPassword: "",
                hiddenTextError: "You must write values for username, password and confirm password"
            });
        }
        event.preventDefault();
    }

    handleKeyPress = (event: any) => {
        if(event.key == 'Enter') {
            this.addUserClick(event);
        }
    };

    handleEscKeyPress = (event: any) => {
        if (event.key == 'Escape') {
            this.backToLoginPageClick();
        }
    };

    render() {
        const createAccountPage = (
            <div className={"create-account-page"} onKeyDown={this.handleEscKeyPress}>
                <h2 id={"create-account-page-text"}>Create an account</h2>
                <div className="create-account-page-warning">
                    Please keep your username free from obscenities and personal information
                </div>
                <form id={"create-account-page-form"}>
                    <input id={"username-create-account-page"} value={this.state.usernameNewAccount} type={"text"}
                           onChange={this.usernameChange} placeholder={"Username"} autoComplete="username"/> <br/>
                    <input id={"password-create-account-page"} value={this.state.passwordNewAccount} type={"password"}
                           onChange={this.passwordChange} placeholder={"Password"} autoComplete="new-password" /> <br/>
                    <input id={"confirm-password-create-account-page"} value={this.state.confirmPassword} type={"password"}
                           onChange={this.confirmPasswordChange} placeholder={"Confirm password"}
                           onKeyDown={this.handleKeyPress} autoComplete="new-password"/> <br/>
                    <span id={"hidden-info-text-create-account-page"}>{this.state.hiddenText}</span>
                    <span id={"hidden-info-error-text-create-account-page"}>{this.state.hiddenTextError}</span> <br/>
                    <Button text={"Create account"} onClick={() => this.addUserClick(event)}/>
                </form>
                <Button text={"Back"} onClick={() => this.backToLoginPageClick()}/>
            </div>
        );

        return (
            this.state.goBack ? <LoginPage /*loggedInState={this.props.loggedInState}*/ /> : createAccountPage
        )
    }
}

export default CreateAccountPage;