import React, { Component } from "react";
import CreateAccountPage from "./CreateAccountPage";
import Button from "../dialogue/Button";
import {Fetch} from "../utils";
import "./LoginPage.css";
import {logIn, logOut} from "../actions/actions";
import {connect} from "react-redux";

interface ILoginPageProps {
    // loggedInState: any;
    loggedIn: boolean,
    logOut: () => void,
    logIn: () => void
}

interface ILoginPageState {
    usernameLogin: string;
    passwordLogin: string;
    isCreating: boolean;
    hiddenText: string;
}

const mapStateToProps = (state: any) => {
    return {
        loggedIn: state.loggedIn
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        logOut: () => dispatch(logOut()),
        logIn: () => dispatch(logIn())
    }
}

class ConnectedLoginPage extends Component<ILoginPageProps, ILoginPageState> {

    state = {
        usernameLogin: "",
        passwordLogin: "",
        isCreating: false,
        hiddenText: ""
    };

    constructor(props: ILoginPageProps) {
        super(props);
        this.usernameChange = this.usernameChange.bind(this);
        this.passwordChange = this.passwordChange.bind(this);
        this.loginClick = this.loginClick.bind(this);
        this.createAccountClick = this.createAccountClick.bind(this);
    }

    usernameChange(event: any) {
        let username = event.target.value;
        this.setState({
            usernameLogin: username.trim()
        });
    }

    passwordChange(event: any) {
        this.handleKeyPress.bind(event);
        let password = event.target.value;
        this.setState({
            passwordLogin: password
        });
    }

    handleKeyPress = (event: any) => {
        if(event.key == 'Enter'){
            this.loginClick(event)
        }
    };

    loginClick(event: any) {
        let username = this.state.usernameLogin;
        let password = this.state.passwordLogin;

        if (username != null && username != "" && password != null && password != "") {

            let account = {username: username, password: password};
            Fetch.post("/api/login", /*user*/ account)
                .then(response => {
                    let version = response.headers.get("Version");
                    Fetch.version = version?version:"";
                    return response.text()
                })
                .then(token => {
                    Fetch.token = token;
                    Fetch.username = username;
                    this.props.logIn();
                    // this.props.loggedInState(true);
                })
                .catch(error=>{
                    console.warn(error);
                    this.setState({
                        hiddenText: error.status + " " + error.statusText
                    });
                })
        }
        event.preventDefault();
    }

    createAccountClick() {
        this.setState({
            isCreating: true
        })
    }

    render() {
        const loginPage = (
            <div className={"login-page"}>
                <form id={"login-page-form"}>
                    <h2 id={"welcome-text-login-page"}>Welcome to MERCHANTS OF ROME</h2>
                    <div className={"login-page-form-inputs"}>
                        <input autoFocus id={"username-login-page"} value={this.state.usernameLogin} onChange={this.usernameChange}
                               type={"text"} placeholder={"Username"} name={"username"} autoComplete="username"/>
                        <input id={"password-login-page"} value={this.state.passwordLogin} onChange={this.passwordChange}
                               type={"password"} placeholder={"Password"} onKeyPress={this.handleKeyPress} name={"current-password"}
                               autoComplete="current-password" />
                    </div>
                    <span id={"hidden-info-text-login-page"}>{this.state.hiddenText}</span> <br/>
                    <Button text={"Login"} onClick={() => this.loginClick(event)}/>
                </form>
                <a id={"go-to-create-account-link"} onClick={this.createAccountClick}>You don't have an account? Click
                    here to create one</a>
            </div>
        );

        return (
            this.state.isCreating ? <CreateAccountPage /*loggedInState={this.props.loggedInState}*/ /> : loginPage
        )
    }
}

const LoginPage = connect(mapStateToProps, mapDispatchToProps)(ConnectedLoginPage);

export default LoginPage;