import React, {Component} from "react";
import {connect} from "react-redux";
import {Time} from "../utils";
import "./MineCave.css";
import {Mine} from "./Mine";
import {Item} from "../Items/Item";
import {collectOre, increaseInventory} from "../actions/actions";

interface IMineCaveProps {
    mine: Mine,
    increaseInventory: (item: Item) => void,
    collectOres: (mines: Mine[]) => void,
    chosen: boolean,
    collect: (mine: Mine) => void
}

interface IMineCaveState {
    timeLeft: number;
}

function mapDispatchToProps(dispatch: any) {
    return {
        increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
        collectOres: (mines: Mine[]) => dispatch(collectOre(mines))
    }
}

class ConnectedMineCave extends Component<IMineCaveProps, IMineCaveState> {
    private interval: any;

    state = {
        timeLeft: 1e9
    };

    constructor(props: IMineCaveProps) {
        super(props);
        this.collect = this.collect.bind(this);
    }

    startInterval(){
        clearInterval(this.interval);
        this.interval = setInterval(() => this.tick(), 1000);
    }

    componentDidMount() {
        if(this.props.mine.timeLeft()>0){
            this.startInterval();
        }
        this.setState({
            timeLeft: this.props.mine.timeLeft()
        });
    }

    componentDidUpdate(prevProps: IMineCaveProps) {
        if (this.state.timeLeft != this.props.mine.timeLeft()) {
            if(this.props.mine.timeLeft()>0){
                this.startInterval();
            }
            this.setState({
                timeLeft: this.props.mine.timeLeft()
            });
        }
    }

    componentWillUnmount() {
        clearInterval(this.interval);
    }

    collect() {
        this.props.collect(this.props.mine);
    }

    tick() {
        if(this.state.timeLeft <=0){
            clearInterval(this.interval);
            return;
        }
        this.setState(state => ({
            timeLeft: state.timeLeft - 1
        }));
    }

    time(){
        let time = Time.timeFormat(this.state.timeLeft);
        if(time == ""){
            return "Finished";
        }
        return time;
    }

    render = () => (
        <div className={"mine-cave"}>
            <div className={"mine-cave-title"}>
                <h5 className={"mine-cave-title-timer"}>{this.time()}</h5>
            </div>
            <div className={`mine-cave-body`} onClick={this.collect}>
                <img className={"mine-cave-body-img"} src={this.props.mine.imageUrl(this.state.timeLeft)} alt={"mine"} />
            </div>
        </div>
    )
}

const MineCave = connect(null, mapDispatchToProps)(ConnectedMineCave);

export default MineCave;