import React, {Component} from "react";
import {Mine} from "./Mine";
import {connect} from "react-redux";
import Button from "../dialogue/Button";
import {Item} from "../Items/Item";
import {collectOre, increaseInventory} from "../actions/actions";
import MineCave from "./MineCave";
import {Fetch} from "../utils";
import {Ore, OreName} from "../Items/Ore";
import {Items, ItemsName} from "../Items/Items";
import {ProcessingPlantFactory} from "../factories/ProcessingFactory";
import {DialogueStandards} from "../dialogue/DialogueStandards";
import {Metal, MetalName} from "../Items/Metal";

enum Dialogue {
    None,
    CollectBefore
}

interface IMountainState {
    chosenMines: number[],
    dialogue: Dialogue
}

interface IMountainProps {
    mines: Mine[],
    collectOres: (mines: Mine[]) => void,
    increaseInventory: (item: Item) => void
}

const mapStateToProps = (state: any) => {
    return {
        mines: state.mines
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
        collectOres: (mines: Mine[]) => dispatch(collectOre(mines))
    }
}

const emptylist: () => number[] = () => [];

class ConnectedMountain extends Component<IMountainProps, IMountainState> {

    state = {
        chosenMines: emptylist(),
        dialogue: Dialogue.None
    };

    constructor(props: IMountainProps) {
        super(props);
        this.collectClick = this.collectClick.bind(this);
        this.mineClick = this.mineClick.bind(this);
    }

    componentDidMount() {}

    componentDidUpdate(prevProps: IMountainProps) {}

    chooseMine(id: number) {
        let chosenMines: number[] = this.state.chosenMines;
        function check() {
            for (let i = 0; i < chosenMines.length; i++) {
                if (chosenMines[i] == id) {
                    chosenMines.splice(i , 1);
                    return true;
                }
            }
            return false;
        }
        if (!check()) {
            chosenMines.push(id);
        }
        this.setState({
            chosenMines: chosenMines
        });
    }

    mineClick(mine: Mine) {
        this.chooseMine(mine.mineId);
        if (mine.stage(mine.timeLeft()) != 0) {
            this.getFromBackend([mine.mineId], [mine]);
        } else {
            this.setState({
                dialogue: Dialogue.CollectBefore
            });
        }
    }

    collectBeforeDialogue() {
        return DialogueStandards.CONFIRM_DIALOGUE(
            "Collecting too early",
            "You can't collect the ores because the miners haven't come up with the ores yet.",
            () => {this.closeDialogue()}
        );
    }

    closeDialogue() {
        this.setState({
            dialogue: Dialogue.None,
            chosenMines: []
        });
    }

    showDialogue() {
        switch (this.state.dialogue) {
            case Dialogue.None:
                return null;
            case Dialogue.CollectBefore:
                return this.collectBeforeDialogue();
        }
    }

    collectClick() {
        let mineIds: number[] = [];
        let mines: Mine[] = [];
        this.props.mines.map(
            (mine: Mine) => {
                if (mine.stage(mine.timeLeft()) != 0) {
                    mineIds.push(mine.mineId);
                    mines.push(mine);
                }
            });
        if (mineIds.length != 0) {
            this.getFromBackend(mineIds, mines);
        }
    }

    private getFromBackend(mineIds: number[], mines: Mine[]) {
        this.props.collectOres(mines);
        Fetch.post("mining/collectfrommine", mineIds).then(response => {

            switch (response.status) {
                case 200:
                    response.json().then(result => {

                        /*let stonesAmount = result[0];
                        let clayAmount = result[1];*/
                        let ironAmount = result[0];
                        /*let copperAmount = result[3];
                        let silverAmount = result[4];
                        let goldAmount = result[5];*/

                        /*let stones = new Items(ItemsName.stone, stonesAmount);
                        let clay = new Items(ItemsName.clay, clayAmount);*/
                        let iron = new Ore(OreName.iron, ironAmount);
                        /*let copper = new Ore(OreName.copper, copperAmount);
                        let silver = new Ore(OreName.silver, silverAmount);
                        let gold = new Ore(OreName.gold, goldAmount);*/

                        /*this.props.increaseInventory(stones);
                        this.props.increaseInventory(clay);*/
                        this.props.increaseInventory(iron);
                        /*this.props.increaseInventory(copper);
                        this.props.increaseInventory(silver);
                        this.props.increaseInventory(gold);*/
                    }).catch(error => {
                        console.error(error);
                    });
                    break;

                default:
                    console.warn(response.status + " " + response.statusText);
            }
        }).catch(error => {
            console.error(error);
        });
    }

    showIronWorker() {
        let name = "foundry";
        let processData = {
            /*clayBricks: {
                input: [new Items(ItemsName.clay, 150)],
                output: [new Items(ItemsName.brick, 1)],
                time: 1200
            },*/
            smeltIronOre: {
                input: [new Ore(OreName.iron, 100)],
                output: [new Metal(MetalName.iron, 1)],
                time: 600,
                show: ()=>true
            }/*,
            smeltCopperOre: {
                input: [new Ore(OreName.copper, 120)],
                output: [new Items(ItemsName.copper, 1)],
                time: 900
            }*//*,
            smeltSilverOre: {
                input: [new Ore(OreName.silver, 200)],
                output: [new Items(ItemsName.silver, 1)],
                time: 1800
            }*//*,
            smeltGoldOre: {
                input: [new Ore(OreName.gold, 280)],
                output: [new Items(ItemsName.gold, 1)],
                time: 2500
            }*/
        };
        return ProcessingPlantFactory.create(name, processData);
    }

    showSmith() {
        let name = "forge";
        let processData = {
            ironSword: {
                input: [new Metal(MetalName.iron, 4)],
                output: [new Metal(MetalName.sword, 1)],
                time: 3600,
                show: ()=>true
            },
            ironNails: {
                input: [new Metal(MetalName.iron, 1)],
                output: [new Metal(MetalName.nails, 5)],
                time: 900,
                show: ()=>false
            }
        };
        return ProcessingPlantFactory.create(name, processData);
    }

    collectButton(){
        return;
        //return this.props.mines.length > 0 ? <Button text={"Collect from all mines"} onClick={this.collectClick}/> : null
    }

    render = () => (
        <div className={"Mountain"}>
            {this.showDialogue()}
            {this.props.mines.map(
                (mine: Mine) =>
                    <MineCave
                        key={mine.mineId}
                        mine={mine}
                        chosen={this.state.chosenMines.includes(mine.mineId)}
                        collect={this.mineClick}
                    />
            )} <br/>
            {this.collectButton()}<br/>
            {this.showIronWorker()}
            {this.showSmith()}
        </div>
    )
}

const Mountain = connect(mapStateToProps, mapDispatchToProps)(ConnectedMountain);

export default Mountain;