import {Component} from "react";
import {DialogueStandards} from "../dialogue/DialogueStandards";
import React from "react";
import {connect} from "react-redux";
import {
    addQuest,
    completeQuest,
    increaseInventory,
    plantSeeds,
    progressQuest,
    setTab,
    unlockSeed
} from "../actions/actions";
import {QuestDialogue, QuestLine} from "./QuestLine";
import {ReduxState} from "../reducers/reducer";
import {FeedbackQuestion, Fetch, HistoryMap, InventoryList} from "../utils";
import {Plant} from "../farming/Plant";
import {Tab} from "../Enum";
import {Item} from "../Items/Item";
import './Quest.css'
import {Mine} from "../mining/Mine";
import {Tree} from "../woodcutting/Tree";

interface IQuestProps{
    questId: number,
    questLine: QuestLine,
    questProgression: number,
    progressQuest: (questId: number) => void,
    completeQuest: (questId: number) => void,
}

interface IQuestState{
    progression: number
}

export interface IQuestData{
    inventory: InventoryList,
    farmPlots: Plant[],
    history: HistoryMap,
    mines: Mine[],
    treePatches: Tree[]
}

export interface IQuestTools{
    plantSeeds: (plant: Plant[]) => void,
    setTab: (tab: Tab) => void,
    increaseInventory: (item: Item) => void,
    unlockSeed: (seed: Plant) => void,
    addQuest: (questId: number) => void,
    loadFeedbackQuestions: (questions: FeedbackQuestion[]) => void,
}

export class QuestFactory{
    static create(questLine: QuestLine): JSX.Element{
        let Quest = connect(this.mstpFactory(questLine.questId), this.mdtpFactory())(ConnectedQuest);
        return <Quest questId={questLine.questId} questLine={questLine} key={questLine.questId}/>
    }

    static mstpFactory = (questId: number) => {
        return (state: any) => {
            return {
                questProgression: state.currentQuests[questId]
            }
        };
    };

    static mdtpFactory = () => {
        return (dispatch: any) => {
            return {
                progressQuest: (questId: number) => dispatch(progressQuest(questId)),
                completeQuest: (questId: number) => dispatch(completeQuest(questId)),
            }
        };
    };


}

class ConnectedQuest extends Component<IQuestProps,IQuestState>{
    state = {
        progression: 0
    };

    constructor(props: IQuestProps) {
        super(props);
        let progression = this.props.questLine.checkpoints[this.props.questProgression];
        //console.log(this.props.questLine.checkpoints,this.props.questProgression);
        this.state = {progression: progression?progression:0}
    }

    show(): JSX.Element{
        //console.log(this.props.questLine);
        return this.convertToDialogue(this.props.questLine.dialogues[this.state.progression])
    }

    onClickConfirmFactory(questDialogue: QuestDialogue){
        return () => {
            if(!questDialogue.requirement()){
                return;
            }
            questDialogue.reward();
            if(questDialogue.checkpoint){
                this.props.progressQuest(this.props.questId);
                Fetch.put(`quests/${this.props.questId}/${this.props.questProgression+1}`)
                    .catch(error=>console.warn(error))
            }
            if(questDialogue.complete){
                this.props.completeQuest(this.props.questId);
                return;
            }
            this.setState({progression: this.state.progression + 1});

        }
    }

    convertToDialogue(questDialogue: QuestDialogue){
        //console.log(questDialogue);
        return DialogueStandards.CONFIRM_DIALOGUE_ELEMENT_BODY(
            questDialogue.title,
            questDialogue.bodyElement,
            this.onClickConfirmFactory(questDialogue),
            false)
    }



    render() {
        return (
            <div className="quest">
                {this.show()}
            </div>

        )
    }


}