import {IQuestData, IQuestTools} from "./Quest";
import {Item} from "../Items/Item";
import {QuestLine} from "./QuestLine";
import React from "react";
import {DialogueTools, InventoryList, InventoryTools, ItemFactory, Strings} from "../utils";
export abstract class QuestBase{
    static questTitle: string = "Quest";
    state: IQuestData;
    tools: IQuestTools;
    protected constructor(state: IQuestData, tools: IQuestTools){
        this.state = state;
        this.tools = tools;
    }

    set(state: IQuestData, tools: IQuestTools){
        this.state = state;
        this.tools = tools;
        return this;
    }

    static image(name: string): JSX.Element{
        return DialogueTools.imageFunction("quest")(name);
    }

    checkItem(name: string, amount:number = 1): boolean{
        return InventoryTools.checkItem(name, amount, this.state.inventory)
    }

    checkItemTotal(category: string, name: string, amount: number = 1): boolean{
        return InventoryTools.checkItemTotal(category, name, amount, this.state.history)
    }

    addItem(name: string, amount: number){
        InventoryTools.addItem(name, amount, this.tools.increaseInventory)
    }

    removeItem(name: string, amount: number){
        InventoryTools.removeItem(name, amount, this.tools.increaseInventory)
    }

    abstract get(): QuestLine
}

export interface IQuestBase{
    new (state: IQuestData, tools: IQuestTools): QuestBase;
    questTitle: string
}