import * as React from "react";

export class QuestLine{
    questId:number;
    dialogues: QuestDialogue[] = [];
    checkpoints: number[] = [0];

    constructor(questId: number){
        this.questId = questId;
    }

    addDialogue(title: string, body: string,
                requirement: ()=>boolean = ()=>true, reward: ()=>void = ()=>{}){
        return this.add(title, body, false, false, requirement, reward);
    }

    addDialogueElementBody(title: string, body: JSX.Element,
                           requirement: ()=>boolean = ()=>true, reward: ()=>void = ()=>{}){
        return this.addElementBody(title, body, false, false, requirement, reward);
    }

    addCheckpoint(title: string, body: string,
                  requirement: ()=>boolean = ()=>true, reward: ()=>void = ()=>{}){
        return this.add(title, body, true, false, requirement, reward);
    }

    addCheckpointElementBody(title: string, body: JSX.Element,
                             requirement: ()=>boolean = ()=>true, reward: ()=>void = ()=>{}){
        return this.addElementBody(title, body, true, false, requirement, reward);
    }

    addFinal(title: string, body: string,
             requirement: ()=>boolean = ()=>true, reward: ()=>void = ()=>{}){
        return this.add(title, body, true, true, requirement, reward);
    }

    addFinalElementBody(title: string, body: JSX.Element,
                        requirement: ()=>boolean = ()=>true, reward: ()=>void = ()=>{}){
        return this.addElementBody(title, body, true, true, requirement, reward);
    }

    private add(title: string, body: string, checkpoint: boolean, complete: boolean,
                requirement: ()=>boolean, reward: ()=>void){
        return this.addElementBody(title,<div><p>{body}</p></div>, checkpoint, complete, requirement, reward)
    }

    private addElementBody(title: string, body: JSX.Element, checkpoint: boolean, complete: boolean,
                           requirement: ()=>boolean, reward: ()=>void){
        let qd = new QuestDialogue(title, body, checkpoint,complete, requirement, reward);
        this.dialogues.push(qd);
        //console.log(qd);
        if(checkpoint){
            this.checkpoints.push(this.dialogues.length);
        }
        return this;
    }
}

export class QuestDialogue{
    title: string;
    bodyElement: JSX.Element;
    checkpoint: boolean;
    complete: boolean;
    requirement: ()=>boolean;
    reward: ()=>void;

    constructor(title: string, bodyElement: JSX.Element, checkpoint: boolean, complete: boolean,
                requirement: ()=>boolean = ()=>true, reward: ()=>void = ()=>{}){
        this.title = title;
        this.bodyElement = bodyElement;
        this.checkpoint = checkpoint;
        this.complete = complete;
        this.requirement = requirement;
        this.reward = reward;
    }
}