import './QuestView.css';
import {Tab} from "../Enum";
import {FeedbackQuestion, Fetch, QuestList} from "../utils";
import {IQuestData, IQuestTools} from "./Quest";
import {Plant} from "../farming/Plant";
import {ReduxState} from "../reducers/reducer";
import {
    addQuest,
    increaseInventory,
    loadFeedbackQuestions,
    plantSeeds,
    setTab,
    setUnlockedSeeds,
    unlockSeed
} from "../actions/actions";
import {Item} from "../Items/Item";
import {Component} from "react";
import {Crop, CropName} from "../Items/Crops";
import {QuestDatabase} from "./QuestDatabase";
import {connect} from "react-redux";
import React from "react";
import QuestThumb from "./QuestThumb";
import {Items, ItemsName} from "../Items/Items";
import {DialogueStandards} from "../dialogue/DialogueStandards";

enum Dialogue{
    None,
    Finished,
}

interface IQuestViewState{
    dialogue: Dialogue,
    questSelected: number
}

interface IQuestViewProps{
    currentQuests: QuestList,
    questData: IQuestData,
    questTools: IQuestTools,
    loaded: boolean
}

const mapStateToProps = (state: ReduxState) => {
    return {
        currentQuests: state.currentQuests,
        questData: {
            inventory: state.inventory,
            farmPlots: state.farmPlots,
            mines: state.mines,
            history: state.history,
            treePatches: state.treePatches
        }
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        questTools: {
            plantSeeds: (plant: Plant[]) => dispatch(plantSeeds(plant)),
            setTab: (tab: Tab) => dispatch(setTab(tab)),
            increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
            unlockSeed: (seed: Plant) => dispatch(unlockSeed(seed)),
            addQuest: (questId: number) => dispatch(addQuest(questId)),
            loadFeedbackQuestions:  (questions: FeedbackQuestion[]) => dispatch(loadFeedbackQuestions(questions)),
        }
    };
}

class ConnectedQuestView extends Component<IQuestViewProps,IQuestViewState>{
    state = {
        dialogue: Dialogue.None,
        questSelected: -1
    };

    constructor(props: IQuestViewProps) {
        super(props);
        this.chooseQuest = this.chooseQuest.bind(this);
    }

    componentDidMount() {
        if(Object.keys(this.props.currentQuests).length===1 && this.state.questSelected==-1){
            this.setState({
                questSelected: Number(Object.keys(this.props.currentQuests)[0])
            });
        }
    }

    componentDidUpdate(prevProps: Readonly<IQuestViewProps>, prevState: Readonly<IQuestViewState>, snapshot?: any): void {
        //console.log(this.state,this.props, prevState, prevProps);
        if(this.props.loaded && Object.keys(this.props.currentQuests).length===0){
            this.props.questTools.setTab(Tab.sellInventory);
            return;
        }
        if(this.props.loaded && Object.keys(this.props.currentQuests).length===1 && this.state.questSelected==-1){
            this.setState({
                questSelected: Number(Object.keys(this.props.currentQuests)[0])
            });
        }
        if(this.state.questSelected >= 0 && this.props.currentQuests[this.state.questSelected] == -1){
            console.log("adjusted");
            this.setState({
                questSelected: -1
            })
        }
    }

    showDialogue(){
        switch(this.state.dialogue){
            case Dialogue.None:
                return null;
            case Dialogue.Finished:
                return this.finishedDialogue();
        }
    }

    finishedDialogue(){
        return DialogueStandards.CONFIRM_DIALOGUE
        (
            "Quest",
            `You have already finished this quest`,
            ()=>this.closeDialogue()
        )
    }

    closeDialogue(){
        this.setState({
            dialogue: Dialogue.None
        })
    }

    showQuest(){
        return QuestDatabase.getElement(this.state.questSelected, this.props.questData, this.props.questTools)
    }

    chooseQuest(id: number){
        if(this.props.currentQuests[id] == -1){
            this.setState({
                dialogue: Dialogue.Finished
            });
            return;
        }
        this.setState({
            questSelected: id
        })
    }


    questlist(){
        return (
            <div className="quest-view-wrapper">
                <div className={"quest-view-quests"}>
                    {Object.keys(this.props.currentQuests)
                    .map((a: string) => Number(a))
                    .map((a: number) => <QuestThumb title={QuestDatabase.getTitle(a)} finished={this.props.currentQuests[a]==-1} id = {a} key={a} chooseQuest={this.chooseQuest}/>)}
                </div>
            </div>
        )
    }


    render() {
        return (
            <div className="quest-view">
                {this.showDialogue()}
                {this.state.questSelected<0?this.questlist():this.showQuest()}
            </div>
        )
    }
}

const QuestView = connect(mapStateToProps, mapDispatchToProps)(ConnectedQuestView);

export default QuestView;
