import {QuestLine} from "../QuestLine";
import React from "react";
import {QuestBase} from "../QuestBase";
import {IQuestData, IQuestTools} from "../Quest";
import {Plant} from "../../farming/Plant";
import {CropName} from "../../Items/Crops";
import {Fetch, Time} from "../../utils";
import {FoodName} from "../../Items/Food";


export class Quest1 extends QuestBase {
    welcomeTitle = "Welcome!";
    static questTitle = "Quest 1";
    constructor(state: IQuestData, tools: IQuestTools) {
        super(state, tools)
    }

    get = () => new QuestLine(1)
        .addDialogue(this.welcomeTitle, "Hello there slave, welcome to the free world!")
        .addDialogueElementBody(this.welcomeTitle,
            <div>
                <p>As a gift for your good servitude I will give you this farm.</p>
                <p>On this farm you now have one patch to grow yourself some food.</p>
                <p>Later on you will be able to expand your farm with more patches to grow food on.</p>
            </div>)
        .addDialogueElementBody(this.welcomeTitle,
            <div>
                <p>I will now guide you through five quests so that you can learn how the free world works.</p>
                <p>Follow me!</p>
            </div>)
        .addCheckpointElementBody(Quest1.questTitle,
            <div>
                <p>I have already planted your first patch of wheat {QuestBase.image("wheat")} </p>
                <p>They should be ready to be harvested quite soon.</p>
            </div> , () => true, ()=>{
                this.tools.plantSeeds([new Plant(CropName.wheat, 0, Time.dateFromTimeLeft(5))])
            })
        .addCheckpointElementBody(Quest1.questTitle,
            <div>
                <p>Please harvest the wheat {QuestBase.image("wheat")} </p>
                <p>You can find the farm plot in the farming tab {QuestBase.image("farming")} </p>
                <p className={"quest-warning-span"}>Note that if you harvest the grain to soon, you will not get any grain.</p>
                <p>Once you have harvested the wheat, come back here to the quest tab {QuestBase.image("quest")} </p>
            </div> , () => this.checkItemTotal("plants", CropName.wheat), ()=>{
                this.tools.unlockSeed(new Plant(CropName.wheat))
            })
        .addCheckpointElementBody(Quest1.questTitle,
            <div>
                <p>Plant more wheat {QuestBase.image("wheatSeeds")} </p>
                <p>You should strive to always have something growing </p>
                <p>There's always a demand for food in this country</p>
            </div> , () => {return this.state.farmPlots.filter((a)=>a.name==CropName.wheat).length>0})
        .addCheckpointElementBody(Quest1.questTitle,
            <div>
                <p>Now, take the wheat {QuestBase.image("wheat")} to the mill {QuestBase.image("mill")}</p>
                <p>You can find the mill in the farming tab {QuestBase.image("farming")} </p>
                <p className={"quest-warning-span"}>You will usually get one flour from 10 wheat, but since you don't own your own mill, the miller takes half of your wheat.</p>
                <p>Then we just have to wait until the miller is finished.</p>
            </div>, () => this.checkItem("flour"))
        .addCheckpointElementBody(Quest1.questTitle,
            <div>
                <p>Take the flour {QuestBase.image("flour")} to the baker {QuestBase.image("baker")}. </p>
                <p>You can find the baker in the farming tab {QuestBase.image("farming")} </p>
                <p className={"quest-warning-span"}>You will usually get one bread for 15 flour, but since you don't own your own bakery, the baker takes half of your flour.</p>
                <p>Then we just have to wait until the baker is finished.</p>
            </div>, () => this.checkItem(FoodName.bread))
        .addCheckpointElementBody(Quest1.questTitle,
            <div>
                <p>Now, sell the bread {QuestBase.image("bread")} to the villagers {QuestBase.image("sellInventory")}.</p>
            </div> , () => (this.checkItem("coins") && !this.checkItem("bread", 1)))
        .addFinalElementBody(Quest1.questTitle,
            <div>
                <p>Congratulations! You have now earned your first coins. </p>
                <p>As an extra reward, here's 5 coins {QuestBase.image("coins")} to aid you on your way</p>
            </div>, () => true, ()=>{
                this.addItem("coins",5);
                this.tools.addQuest(2);
                setTimeout(()=>{
                    Fetch.get("feedback/questions")
                        .then(response => response.json())
                        .then(questions => this.tools.loadFeedbackQuestions(questions))
                }, 10000);

        });
}