import React, {Component} from 'react';
import './Scoreboard.css';
import {DialogueTools, Fetch, Strings} from '../utils'
import {Item} from "../Items/Item";
import {ScoreboardTab} from "./ScoreboardTab";
import {ScoreboardEntry} from "./ScoreboardEntry";

export class ScoreboardData {
    [key: string]: number
}

export interface ScoreboardTabData{
    name: string,
    description:string,
    image: string,
}

interface ScoreboardInfo{
    [key: string]: ScoreboardTabData
}

const scoreboardInfo: ScoreboardInfo = {
    fortune: {
        name: "fortune",
        description: "Total value of all assets",
        image: "trophy1"
    },
    coins: {
        name: "coins",
        description: "Current amount of coins",
        image: "coins"
    },
    wheatHarvested: {
        name: "wheatHarvested",
        description: "Total amount of wheat harvested",
        image: "wheat"
    },
};

interface IScoreboardState {
    scoreboard: ScoreboardData
    tab: ScoreboardTabData
}

interface IScoreboardProps {
}

export class Scoreboard extends Component<IScoreboardProps, IScoreboardState> {
    state = {
        scoreboard: new ScoreboardData(),
        tab: scoreboardInfo.fortune,
    };

    constructor(props: IScoreboardProps) {
        super(props);
        this.setTab = this.setTab.bind(this);
    }

    componentDidMount() {
        Fetch.get("accounts/scoreboard/" + this.state.tab.name)
            .then(response => response.json())
            .then(scoreboard => {
                this.setState({scoreboard})
            })
    }

    setTab(tabName: string){
        this.setState({
            tab: scoreboardInfo[tabName]
        });
        Fetch.get("accounts/scoreboard/" + tabName)
            .then(response => response.json())
            .then(scoreboard => {
                this.setState({scoreboard})
            })
    }

    render() {
        return (
            <div className={"scoreboard"}>
                <div className={"scoreboard-buttons-wrapper"}>
                    <div className="scoreboard-buttons">
                        {Object.keys(scoreboardInfo)
                            .map((tab: string, index: number) => <ScoreboardTab scoreboardTab={scoreboardInfo[tab]} key={index} selected={this.state.tab.name == tab} setTab={this.setTab} />)}
                    </div>
                </div>
                <span className={"scoreboard-description"}>{this.state.tab.description}</span>
                <div className={"scoreboard-list"}>
                {Object.keys(this.state.scoreboard)
                    .sort((a: string, b: string) => {
                        return this.state.scoreboard[b] - this.state.scoreboard[a]
                    })
                    .map((username: string, index: number) => <ScoreboardEntry username={username} index={index} score = {this.state.scoreboard[username]} key={index}/>)}
                </div>
            </div>

        )
    }
}