import React, {Component} from "react";
import {connect} from "react-redux";
import ChangePasswordPage from "./ChangePasswordPage";
import Button from "../dialogue/Button";
import {DialogueTools, Fetch} from "../utils";
import "./Settings.css";
import App from "../App";
import {logIn, logOut} from "../actions/actions";
import LoginPage from "../login/LoginPage";
import {DialogueStandards} from "../dialogue/DialogueStandards";

interface ISettingsState {
    oldPassword: string;
    newPassword: string;
    confirmPassword: string;
    hiddenText: string;
    hiddenTextError: string;
    showingRules: boolean
}

interface ISettingsProps {
    loggedIn: boolean,
    logOut: () => void,
    logIn: () => void,
    acceptRules: ()=> void,
    rulesAccepted: boolean,
}

const mapStateToProps = (state: any) => {
    return {
        loggedIn: state.loggedIn
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        logOut: () => dispatch(logOut()),
        logIn: () => dispatch(logIn())
    }
}

class ConnectedSettings extends Component<ISettingsProps, ISettingsState> {

    state = {
        oldPassword: "",
        newPassword: "",
        confirmPassword: "",
        hiddenText: "",
        hiddenTextError: "",
        showingRules: false,
    };

    constructor(props: ISettingsProps) {
        super(props);
        this.logoutClick = this.logoutClick.bind(this);
        this.oldPasswordChange = this.oldPasswordChange.bind(this);
        this.newPasswordChange = this.newPasswordChange.bind(this);
        this.confirmPasswordChange = this.confirmPasswordChange.bind(this);
        this.changePasswordClick = this.changePasswordClick.bind(this);
    }

    oldPasswordChange(event: any) {
        let password = event.target.value;
        this.setState({
            oldPassword: password
        });
    }

    newPasswordChange(event: any) {
        let password = event.target.value;
        this.setState({
            newPassword: password
        });
    }

    confirmPasswordChange(event: any) {
        let password = event.target.value;
        this.setState({
            confirmPassword: password
        });
    }

    changePasswordClick() {
        let oldPassword = this.state.oldPassword;
        let newPassword = this.state.newPassword;
        let confirmPassword = this.state.confirmPassword;

        if (oldPassword != null && oldPassword != "" && newPassword != null && newPassword != "" && confirmPassword != null && confirmPassword != "") {
            if (newPassword === confirmPassword) {

                Fetch.put("/api/accounts/changepassword", [oldPassword, newPassword]).then(response => {
                    switch (response.status) {
                        case 200:
                            this.setState({
                                oldPassword: "",
                                newPassword: "",
                                confirmPassword: "",
                                hiddenText: "The password has changed",
                                hiddenTextError: ""
                            });
                            break;
                        default:
                            this.setState({
                                oldPassword: "",
                                newPassword: "",
                                confirmPassword: "",
                                hiddenTextError: response.status + " " + response.statusText,
                                hiddenText: ""
                            });
                            break;
                    }
                }).catch(error => {
                    console.log(error);
                    this.setState({
                        oldPassword: "",
                        newPassword: "",
                        confirmPassword: "",
                        hiddenTextError: "Something went wrong",
                        hiddenText: ""
                    });
                });
            } else {
                this.setState({
                    oldPassword: "",
                    newPassword: "",
                    confirmPassword: "",
                    hiddenText: "",
                    hiddenTextError: "Passwords don't match"
                });
            }
        } else {
            this.setState({
                oldPassword: "",
                newPassword: "",
                confirmPassword: "",
                hiddenText: "",
                hiddenTextError: "You must write values for username, password and confirm password"
            });
        }
    }

    handleKeyPress = (event: any) => {
        if(event.key == 'Enter'){
            this.changePasswordClick()
        }
    };

    logoutClick() {
        console.log("Log out clicked");
        this.props.logOut();
    }

    showRules(){
        if(this.state.showingRules){
            return DialogueStandards.YES_NO_DIALOGUE_ELEMENT_BODY(
                "Rules",
                <div className={"rule-dialogue-text"}>
                    <p>To play this game you have to accept these rules, and follow them</p>
                    <p>More rules might be added and rules might be adjusted in the future,
                        at which point we might require you to agree to the rules again</p>
                    <ul>
                        <li>
                            No scripting. This includes every version of progressing in the game that
                            is not a human being clicking on the different elements in the game. Breaking this rule can
                            lead to the suspension of your account.
                        </li>
                        <li>
                            Your username should be free from obscenities and personal information.
                            If they are not, we reserve the right to censor how your name is displayed in the high scores
                            at our discretion. If we have censored your name, you may contact us through discord or otherwise
                            to request a custom (rule abiding) name yourself. Note that these name changes do not
                            change which username is used to log in.
                        </li>
                        <li>
                            Intentionally exploiting bugs is not allowed. If you find an exploitable bug,
                            report it either on Reddit or Discord, and don't use it to get an advantage.
                        </li>
                    </ul>
                </div>,
                ()=>{this.props.acceptRules(); this.setState({showingRules:false})},
                ()=>{if(this.props.rulesAccepted){this.setState({showingRules:false})}}
            )
        }
    }

    render = () => (
        <div className={"Settings"}>
            <div className={"rules-button"} onClick={()=>this.setState({showingRules: true})}>
                <span className="rules-button-text">Rules</span>
                {DialogueTools.imageFunction("rules")("rules")}

            </div>
            {this.showRules()}
            <div className={"redirect"}>
                <Button onClick={()=>window.open('https://www.reddit.com/r/MerchantsOfRome/comments/b63n66/bug_submissions/', '_blank')} text={"Bug Submissions"}/>
                <Button onClick={()=>window.open('https://www.reddit.com/r/MerchantsOfRome/comments/bavr71/content_suggestions/', '_blank')} text={"Content Suggestions"}/>
            </div>
            <div className={"change-password"}>
                <h2 id={"change-password-page-text"}>Change password</h2>
                <form id={"change-password-page-form"}>
                    <input id={"old-password-input-change-password-page"} value={this.state.oldPassword}
                           type={"password"} onChange={this.oldPasswordChange} placeholder={"Old password"}
                            autoComplete="current-password"/> <br/>
                    <input id={"new-password-input-change-password-page"} value={this.state.newPassword}
                           type={"password"} onChange={this.newPasswordChange} placeholder={"New password"}
                           autoComplete="new-password"/> <br/>
                    <input id={"confirm-password-input-change-password-page"} value={this.state.confirmPassword}
                           type={"password"} onChange={this.confirmPasswordChange} placeholder={"Confirm password"}
                           onKeyPress={this.handleKeyPress} autoComplete="new-password"/> <br/>
                    <span id={"hidden-info-text-change-password-page"}>{this.state.hiddenText}</span>
                    <span id={"hidden-info-error-text-change-password-page"}>{this.state.hiddenTextError}</span> <br/>
                    <Button text={"Change password"} onClick={this.changePasswordClick}/>
                </form>
            </div>
            <iframe className="discord-widget" src={`https://discordapp.com/widget?id=557476679748943875&theme=dark&username=${Fetch.username}`}
                    width="350" height="350" frameBorder={0} />
            <Button text={"Log out"} onClick={this.logoutClick}/>
        </div>
    )
}

const Settings = connect(mapStateToProps, mapDispatchToProps)(ConnectedSettings);

export default Settings;