import {Items, ItemsName} from "../Items/Items";
import {Fruit, FruitName} from "../Items/Fruit";
import {Wood, WoodName} from "../Items/Wood";


abstract class TreeDatabase{
    static data = {
        empty: {
            timeToGrow: -1,
            timeToGrowFruit: -1,
            stages: 1,
            logs: WoodName.logs,
            fruit: FruitName.apple
        },
        appleTree: {
            timeToGrow: 12*3600,
            timeToGrowFruit: 3600,
            stages: 3,
            logs: WoodName.logs,
            fruit: FruitName.apple
        },
        peachTree: {
            timeToGrow: 16*3600,
            timeToGrowFruit: 2*3600,
            stages: 3,
            logs: WoodName.logs,
            fruit: FruitName.peach
        },
        orangeTree: {
            timeToGrow: 24*3600,
            timeToGrowFruit: 1800,
            stages: 3,
            logs: WoodName.logs,
            fruit: FruitName.orange
        },
        chestnutTree: {
            timeToGrow: 8*3600,
            timeToGrowFruit: 4*3600,
            stages: 3,
            logs: WoodName.logs,
            fruit: FruitName.chestnut
        }
    };
}

export enum TreeName{
    empty = "empty",
    appleTree = "appleTree",
    peachTree = "peachTree",
    orangeTree = "orangeTree",
    chestnutTree = "chestnutTree",

}

export class Tree{
    name: TreeName;
    timeToGrow: number;
    timeToGrowFruit: number;
    stages: number;
    finishTime: Date;
    finishTimeFruit: Date;
    patchId: number;
    logs: WoodName;
    fruit: FruitName;

    constructor(name: TreeName, patchId: number = -1, finishTime?: Date, finishTimeFruit?: Date) {
        let treeData = TreeDatabase.data[name];
        this.timeToGrow = treeData.timeToGrow;
        this.timeToGrowFruit = treeData.timeToGrowFruit;
        this.stages = treeData.stages;
        this.logs =treeData.logs;
        this.fruit = treeData.fruit;
        this.name = name;
        this.patchId = patchId;
        if(finishTime){
            this.finishTime = finishTime
        }else{
            let finishDate = new Date();
            finishDate.setSeconds(finishDate.getSeconds()+this.timeToGrow);
            this.finishTime = finishDate;
        }
        if(finishTimeFruit){
            this.finishTimeFruit = finishTimeFruit
        }else{
            let finishDateFruit = new Date(this.finishTime.getTime());
            finishDateFruit.setSeconds(finishDateFruit.getSeconds()+this.timeToGrowFruit);
            this.finishTimeFruit = finishDateFruit;
        }

    }

    timeLeft(){
        return Math.max(0,Math.round((this.finishTime.getTime()-new Date().getTime())/1000));
    }

    timeLeftFruit(){
        return Math.max(0,Math.round((this.finishTimeFruit.getTime()-new Date().getTime())/1000));
    }

    imageUrl(time: number, fullyGrown: boolean, fullyGrownFruit: boolean){
        return `images/woodcutting/${this.name}${this.stage(time, fullyGrown, fullyGrownFruit)}.png`
    }

    stage(time: number, fullyGrown: boolean, fullyGrownFruit: boolean){

        if(fullyGrownFruit)
            return this.stages;
        if(fullyGrown)
            return this.stages-1;
        return Math.max(Math.min((this.stages-1)-Math.ceil(time/(this.timeToGrow/(this.stages-2))),this.stages-1),0);
    }

    finished(time: number){
        return time>this.timeToGrow;
    }

    harvestFruit(): Fruit{
        return new Fruit(this.fruit)
    }

    chop(){
        return new Wood(this.logs)
    }
}