import React, {Component} from 'react';
import {Tree} from "./Tree";
import {Strings, Time} from "../utils";
import './TreePatch.css';
import {Item} from "../Items/Item";
import {CropName} from "../Items/Crops";
import {chopTree, harvestFruit, increaseInventory, plantSeeds, plantTrees, uprootSeed} from "../actions/actions";
import {connect} from "react-redux";

interface ITreePatchProps {
    tree: Tree
    increaseInventory: (item: Item) => void
    chosen: boolean
    plotClick: (tree: Tree, finished: boolean, finishedFruit: boolean) => void
}

interface ITreePatchState {
    timeLeft: number
    fullyGrown: boolean
}

function mapDispatchToProps(dispatch: any) {
    return {
        increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
    };
}

class ConnectedTreePatch extends Component<ITreePatchProps, ITreePatchState> {
    private interval: any;
    state = {
        timeLeft: 1e9,
        fullyGrown: false,
    };

    constructor(props: ITreePatchProps) {
        super(props);
        this.click = this.click.bind(this);
    }

    componentDidMount() {
        if(this.props.tree.timeLeft() > 0 || this.props.tree.timeLeftFruit() > 0){
            this.startInterval();
        }
        //fetch time
        let timeLeft = Time.hasHappened(this.props.tree.finishTime) ? this.props.tree.timeLeftFruit() : this.props.tree.timeLeft();
        this.setState({
            timeLeft,
            fullyGrown: Time.hasHappened(this.props.tree.finishTime)
        });
    }

    startInterval(){
        clearInterval(this.interval);
        this.interval = setInterval(() => this.tick(), 1000);
    }

    componentDidUpdate(prevProps: ITreePatchProps) {
        //console.log("update", this.state, prevProps);
        if (prevProps.tree.name != this.props.tree.name) {
            if(!Time.hasHappened(this.props.tree.finishTime)){
                this.startInterval();
                this.setState({
                    timeLeft: this.props.tree.timeLeft(),
                    fullyGrown: false
                })
            }else{
                if(!Time.hasHappened(this.props.tree.finishTimeFruit)){
                    this.startInterval();
                }
                this.setState({
                    timeLeft: this.props.tree.timeLeftFruit(),
                    fullyGrown: true
                })
            }
        }else if (prevProps.tree.finishTimeFruit != this.props.tree.finishTimeFruit) {
            if(!Time.hasHappened(this.props.tree.finishTimeFruit)){
                this.startInterval();
            }
            this.setState({
                timeLeft: this.props.tree.timeLeftFruit()
            })
        }
    }

    fullyGrownFruit(){
        return this.state.fullyGrown && this.state.timeLeft <= 0
    }

    click() {
        this.props.plotClick(this.props.tree, this.state.fullyGrown, this.fullyGrownFruit());
    }

    tick() {
        if (this.props.tree.name == "empty"){
            clearInterval(this.interval);
            return;
        }
        if(this.state.timeLeft === 1 && !this.state.fullyGrown){
            this.setState({
                fullyGrown: true,
                timeLeft: this.props.tree.timeLeftFruit()
            });
            return;
        }
        if(this.state.timeLeft <=0){
            clearInterval(this.interval);
            return;
        }
        this.setState(state => ({
            timeLeft: state.timeLeft - 1
        }));
    }

    componentWillUnmount() {
        clearInterval(this.interval);
    }

    render() {
        return (
            <div className="tree-patch">
                <div className="tree-patch-title">
                    <h4 className="tree-patch-title-name">{Strings.decamelize(this.props.tree.name)}</h4>
                    <h5 className="tree-patch-title-timer">{Time.timeFormat(this.state.timeLeft)}</h5>
                </div>
                <div className={`tree-patch-body${this.props.chosen ? " selected" : ""}`} onClick={this.click}>
                    <img className="tree-patch-body-img" src={this.props.tree.imageUrl(this.state.timeLeft, this.state.fullyGrown, this.fullyGrownFruit())}
                         alt={this.props.tree.name}/>
                </div>
            </div>
        )
    }
}

const TreePatch = connect(null, mapDispatchToProps)(ConnectedTreePatch);

export default TreePatch;
